Package: archeofrag
Type: Package
Title: Spatial Analysis in Archaeology from Refitting Fragments
Version: 1.2.4
Date: 2026-04-22
Authors@R: person("Sebastien", "Plutniak", email = "sebastien.plutniak@posteo.net", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-6674-3806"))
Author: Sebastien Plutniak [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6674-3806>)
Maintainer: Sebastien Plutniak <sebastien.plutniak@posteo.net>
Description: Methods to analyse spatial units in archaeology from the refitting relationships between fragments of objects scattered in these units (e.g. stratigraphic layers). Graphs are used to model archaeological observations. The package is mainly based on the 'igraph' package for graph analysis. Functions can: 1) create, manipulate, visualise, and simulate fragmentation graphs, 2) measure the cohesion and admixture of archaeological spatial units, and 3) characterise the topology of a specific set of refitting relationships. A series of published empirical datasets is included. Documentation about 'archeofrag' is provided by a vignette and by the accompanying scientific papers: Plutniak (2021, Journal of Archaeological Science, <doi:10.1016/j.jas.2021.105501>) and Plutniak (2022, Journal of Open Source Software, <doi:10.21105/joss.04335>). This package is complemented by the 'archeofrag.gui' R package, a companion GUI application available at <https://analytics.huma-num.fr/Sebastien.Plutniak/archeofrag/>.
License: GPL-3
Repository: CRAN
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.5.0)
Imports: igraph, graphics, stats, grDevices, methods, utils
Suggests: RBGL, knitr, covr, rmarkdown, markdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
URL: https://github.com/sebastien-plutniak/archeofrag
BugReports: https://github.com/sebastien-plutniak/archeofrag/issues
NeedsCompilation: no
Packaged: 2026-04-21 22:42:47 UTC; seb
Date/Publication: 2026-04-22 07:10:09 UTC
Built: R 4.5.2; ; 2026-04-22 09:38:36 UTC; unix

Package: archeofrag
Type: Package
Title: Spatial Analysis in Archaeology from Refitting Fragments
Version: 1.2.4
Date: 2026-04-22
Authors@R: person("Sebastien", "Plutniak", email = "sebastien.plutniak@posteo.net", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-6674-3806"))
Author: Sebastien Plutniak [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6674-3806>)
Maintainer: Sebastien Plutniak <sebastien.plutniak@posteo.net>
Description: Methods to analyse spatial units in archaeology from the refitting relationships between fragments of objects scattered in these units (e.g. stratigraphic layers). Graphs are used to model archaeological observations. The package is mainly based on the 'igraph' package for graph analysis. Functions can: 1) create, manipulate, visualise, and simulate fragmentation graphs, 2) measure the cohesion and admixture of archaeological spatial units, and 3) characterise the topology of a specific set of refitting relationships. A series of published empirical datasets is included. Documentation about 'archeofrag' is provided by a vignette and by the accompanying scientific papers: Plutniak (2021, Journal of Archaeological Science, <doi:10.1016/j.jas.2021.105501>) and Plutniak (2022, Journal of Open Source Software, <doi:10.21105/joss.04335>). This package is complemented by the 'archeofrag.gui' R package, a companion GUI application available at <https://analytics.huma-num.fr/Sebastien.Plutniak/archeofrag/>.
License: GPL-3
Repository: CRAN
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.5.0)
Imports: igraph, graphics, stats, grDevices, methods, utils
Suggests: RBGL, knitr, covr, rmarkdown, markdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
URL: https://github.com/sebastien-plutniak/archeofrag
BugReports: https://github.com/sebastien-plutniak/archeofrag/issues
NeedsCompilation: no
Packaged: 2026-04-21 22:42:47 UTC; seb
Date/Publication: 2026-04-22 07:10:09 UTC
Built: R 4.6.0; ; 2026-04-26 02:40:09 UTC; unix

export(frag.cycles)
export(frag.diameters)
export(frag.edges.weighting)
export(frag.get.layers)
export(frag.get.layers.pair)
export(frag.graph.plot)
export(frag.graph.reduce)
export(frag.layers.admixture)
export(frag.layers.cohesion)
export(frag.get.parameters)
export(frag.observer.failure)
export(frag.path.lengths)
export(frag.relations.by.layers)
export(frag.simul.process)
export(frag.simul.compare)
export(frag.cohesion.ranking)
export(frag.simul.summarise)
export(make_cr_graph)
export(make_sr_graph)
export(make_crsr_graph)
export(make_frag_object)
exportClasses(Frag.object)
importFrom(grDevices, rgb)
importFrom(graphics, plot, hist)
importFrom(methods, new)
importFrom(utils, combn)
importFrom(stats, na.omit, sd, wilcox.test, median, mad)
importFrom(igraph,
	add_edges, 
	add_layout_, 
	add_vertices, 
	as_edgelist, 
	as_graphnel,
	articulation_points,
	bibcoupling, 
	components, 
	component_wise, 
	complementer,
	"%--%",
	decompose, 
	degree,
	delete_graph_attr,
	delete_vertex_attr, 
	delete_vertices, 
	diameter, 
	distance_table, 
	E, 
	edge_attr, 
	ego,
	gorder, 
	graph_attr, 
	graph_from_adjacency_matrix, 
	graph_from_data_frame, 
	gsize, 
	induced_subgraph, 
	is_igraph, 
	is_directed,
	layout_with_fr, 
	make_ring, 
	make_graph,
	plot.igraph, 
	set_graph_attr, 
	set_vertex_attr, 
	simplify, 
	subgraph_from_edges, 
	subgraph_isomorphisms, 
	transitivity, 
	union,
	V, 
	vertex_attr, 
	vertex_attr_names, 
	with_fr) 

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  fig.path = "man/figures/",
  comment = "#>"
)

## ----make-empirical-graph,  message=F-----------------------------------------
library(archeofrag)
abu.frag <- make_frag_object(cr=liangabu.connection, fragments=liangabu.fragments)
abu.g <- make_cr_graph(abu.frag)

## ----frag.relations.by.layers-------------------------------------------------
frag.relations.by.layers(abu.g, "layer")

## ----manipulate-plot-abu, fig.align="center"----------------------------------
par(mar=c(1, 0, 2, 0))
frag.graph.plot(abu.g, layer.attr="layer", main="All layers")

## ----manipulate-get.layer.pair------------------------------------------------
abu.g12 <- frag.get.layers.pair(abu.g, layer.attr="layer",
                                sel.layers=c("1", "2"))

## ----manipulate-plot-abu2, fig.align="center"---------------------------------
par(mar=c(1, 0, 2, 0))
frag.graph.plot(abu.g12, layer.attr="layer", main="Layers 1 and 2")

## ----manipulate-get.layer.pair2-----------------------------------------------
frag.get.layers.pair(abu.g, layer.attr="layer", sel.layers=c("1", "2"),
                     size.mini=2, mixed.components.only=TRUE)

## ----manipulate-frag.get.layers-----------------------------------------------
frag.get.layers(abu.g, layer.attr="layer", sel.layers="1")

## ----weights-g1---------------------------------------------------------------
abu.g12 <- frag.edges.weighting(abu.g12, layer.attr="layer")

## ----cohesion-g12-------------------------------------------------------------
frag.layers.cohesion(abu.g12, layer.attr="layer")

## ----weights-g12morpho--------------------------------------------------------
abu.g12morpho <- frag.edges.weighting(abu.g12,
                                      layer.attr="layer",
                                      morphometry="length")

## ----cohesion-g12morpho-------------------------------------------------------
frag.layers.cohesion(abu.g12morpho, layer.attr="layer")

## ----admixture----------------------------------------------------------------
# topology-based weighting:
frag.layers.admixture(abu.g12, layer.attr="layer")
# topology + morphometry weighting:
frag.layers.admixture(abu.g12morpho, layer.attr="layer")

## ----make-simulatd-graph, message=FALSE---------------------------------------
simul.g <- frag.simul.process(n.components=20, vertices=50)

## ----simulator-all-params, eval=F---------------------------------------------
# frag.simul.process(initial.layers=1,
#                    n.components=20,
#                    vertices=50,
#                    edges=40,
#                    balance=.4,
#                    components.balance=.4,
#                    disturbance=.1,
#                    aggreg.factor=0,
#                    planar=FALSE,
#                    asymmetric.transport.from="1")

## ----frag.observer.failure, eval=FALSE----------------------------------------
# frag.observer.failure(abu.g12, likelihood=0.2)

## ----params-------------------------------------------------------------------
params <- frag.get.parameters(abu.g12, layer.attr="layer")
params

## ----simulator-test-----------------------------------------------------------
# for H2:
test.2layers.g <- frag.simul.process(initial.layers=2,
                                    n.components=params$n.components,
                                    vertices=params$vertices,
                                    disturbance=params$disturbance,
                                    aggreg.factor=params$aggreg.factor,
                                    planar=params$planar)
# for H1:
test.1layer.g <- frag.simul.process(initial.layers=1,
                                   n.components=params$n.components,
                                   vertices=params$vertices,
                                   disturbance=params$disturbance,
                                   aggreg.factor=params$aggreg.factor,
                                   planar=params$planar)

## ----simulator-test2, message=FALSE-------------------------------------------
run.test2 <- function(x){
  frag.simul.process(initial.layers=2, # note the different value
                     n.components=params$n.components,
                     vertices=params$vertices,
                     disturbance=params$disturbance,
                     aggreg.factor=params$aggreg.factor,
                     planar=params$planar)
}

## ----simulator-test2-run, message=FALSE---------------------------------------
test2.results <- lapply(1:100, run.test2)

## ----manipulate-param-plot-mar, echo=FALSE------------------------------------
par(mar=c(5, 4, 4, 2))

## ----simulator-test2-edges, message=FALSE, fig.align="center", fig.width=4, fig.height=3----
edges.res <- sapply(test2.results,
                    function(g) frag.get.parameters(g, "layer")$edges)
plot(stats::density(edges.res), main="Edges")
abline(v=params$edges, col="red")

## ----simulator-test2-admix, message=FALSE, fig.align="center", fig.width=4, fig.height=3----
admix.res <- sapply(test2.results,
                    function(g) frag.layers.admixture(g, "layer"))
plot(stats::density(admix.res), main="Admixture")
abline(v=frag.layers.admixture(abu.g12, "layer"), col="red")

## ----simul-compare, message=FALSE---------------------------------------------
compare.res <- frag.simul.compare(abu.g12, layer.attr="layer",
                                  iter=30, summarise=FALSE)
head(compare.res$h1.data)

## ----simul-summarise, message=FALSE-------------------------------------------
frag.simul.summarise(abu.g12, layer.attr="layer",
                     compare.res$h1.data,
                     compare.res$h2.data)

## ----make-similarity----------------------------------------------------------
# make a frag object and generate a similarity graph:
abu.frag <- make_frag_object(sr=liangabu.similarity, fragments=liangabu.fragments)
abu.sr <- make_sr_graph(abu.frag)

## ----count-similarity---------------------------------------------------------
# count of similarity relationships in and between layers:
simil.by.layers.df <- frag.relations.by.layers(abu.sr, "layer")
simil.by.layers.df

## ----similarity-perc-tab------------------------------------------------------
# percentage of similarity relationships in and between layers:
round(simil.by.layers.df / sum(simil.by.layers.df, na.rm=T) * 100, 0)

## ----similarity-dendr,  eval=T------------------------------------------------
# turn similarity into distance:
simil.dist <- max(c(simil.by.layers.df), na.rm=T) - simil.by.layers.df
simil.dist <- as.dist(simil.dist)
# hierarchical clustering:
clust.res <- stats::hclust(simil.dist, method="ward.D2")

## ----similarity-dendr-fig, fig.width = 3, fig.height = 3, fig.align="center", fig.cap="Hierarchical clustering of the pottery layers in Liang Abu (distance: based on the number of similarity relationships; clustering method: Ward).", eval=T, message=F----
clust.res$labels <- as.character(factor(clust.res$labels, 
                                        levels=c("0", "1", "2"),
                                        labels=c("layer 0", "layer 1", "layer 2")))
plot(clust.res, hang=-1, axes=F, ann=F)

## ----subgraphs-simul----------------------------------------------------------
# simulate a fragmentation graph:
simul.g <- frag.simul.process(initial.layers=2,
                              n.components=20,
                              vertices=70,
                              balance=.45)
# extract the subgraph of each spatial unit:
simul.g1 <- frag.get.layers(simul.g, layer.attr="layer", sel.layers="1")[[1]]
simul.g2 <- frag.get.layers(simul.g, layer.attr="layer", sel.layers="2")[[1]]

## ----cycles-simul1------------------------------------------------------------
rbind(
  "unit1" = frag.cycles(simul.g1, kmax=5),
  "unit2" = frag.cycles(simul.g2, kmax=5))

## ----pathlengths--------------------------------------------------------------
frag.path.lengths(simul.g1)
frag.path.lengths(simul.g2)
frag.path.lengths(simul.g2, cumulative=T)

## ----diameters----------------------------------------------------------------
frag.diameters(simul.g1)
frag.diameters(simul.g2)

.remove.vertice.not.conserving.components.nr <- function(g, final.frag.nr) {
  # exclude protected vertices if any:
  vertice.selection <- ! igraph::V(g)$preserve  
  
  if ( ! sum(vertice.selection)) {
    g$is.reducible <- FALSE 
    return(g)
  }
  vertice.selection <- sample(which(vertice.selection), 1)
  selected.vertice.to.delete <- igraph::V(g)[vertice.selection]
    
  # delete on a copy of the graph:
  g2 <- igraph::delete_vertices(g, selected.vertice.to.delete)
  g2 <- igraph::delete_vertices(g2, igraph::V(g2)[igraph::degree(g2) == 0 & ! igraph::V(g2)$preserve]) # delete singletons

  # delete on the actual graph:
  if (igraph::gorder(g2) >= final.frag.nr) {
    g <- igraph::delete_vertices(g, selected.vertice.to.delete)
    g <- igraph::delete_vertices(g, igraph::V(g)[igraph::degree(g) == 0 & ! igraph::V(g)$preserve])
  } else{ igraph::V(g)[vertice.selection]$preserve <- TRUE } # exclude the selected vertex in next iterations
  g
}

.remove.vertice.conserving.components.nr <- function(g) {
  # select and delete a vertex if it does not decrease the number of connected components
  # (exclude vertices in dyadic components, articulations points, and with edges between spatial units:
  igraph::V(g)$component.id <- igraph::components(g)$membership
  not.dyadic <- names(table(igraph::V(g)$component.id)[table(igraph::V(g)$component.id) > 2])
  
  vertice.selection <- (igraph::V(g)$component.id %in% not.dyadic) &
    ( ! igraph::V(g) %in% igraph::articulation_points(g) ) & 
    ( ! igraph::V(g)$preserve )
  
  if (sum(vertice.selection)) {
    selected.vertice.to.delete <- sample(igraph::V(g)[vertice.selection], 1)
    g <- igraph::delete_vertices(g, selected.vertice.to.delete)
  } else{ g$is.reducible <- FALSE }
  g
}


.reduce.graph <- function(graph, final.frag.nr, conserve.objects.nr){
  graph$is.reducible <- TRUE
  
  if(conserve.objects.nr){
    while ((igraph::gorder(graph) > final.frag.nr) & graph$is.reducible) {
      graph <- .remove.vertice.conserving.components.nr(graph)
    }
  } else {
    while ((igraph::gorder(graph) > final.frag.nr) & graph$is.reducible) {
      graph <- .remove.vertice.not.conserving.components.nr(graph, final.frag.nr)
    }
  }
  igraph::delete_graph_attr(graph, "is.reducible")
}


.concatenate.attr <- function(graph, attribute){
  attr <- c(igraph::vertex_attr(graph, paste0(attribute, "_1")), 
            igraph::vertex_attr(graph, paste0(attribute, "_2")) 
  )
  attr[ ! is.na(attr)]
}


frag.graph.reduce <- function(graph = NULL, n.frag.to.remove = NULL, conserve.objects.nr = FALSE, conserve.fragments.balance = FALSE, conserve.inter.units.connection = FALSE, verbose = FALSE) {
  .check.frag.graph(graph)
  if( ! is.logical(conserve.objects.nr)){stop("'conserve.objects.nr' must be TRUE or FALSE.")}
  if( ! is.numeric(n.frag.to.remove)){stop("'n.frag.to.remove' must be an integer value.")}
  
  igraph::V(graph)$preserve <- FALSE
  
  v1 <- igraph::V(graph)$layer == sort(unique(igraph::V(graph)$layer))[1] 
  v2 <- igraph::V(graph)$layer == sort(unique(igraph::V(graph)$layer))[2] 
  inter.units.edges <- igraph::E(graph)[ v1 %--% v2 ]
  
  # tag vertices with inter-unit edges to keep:
  if(conserve.inter.units.connection){
    proportion <- (igraph::gorder(graph) - n.frag.to.remove) / igraph::gorder(graph)
    rows <- floor(length(inter.units.edges) * proportion)
    inter.units.edges <- inter.units.edges[sample(seq_len(length(inter.units.edges)), rows), ]
    v.to.preserve <- unique(c(igraph::as_edgelist(graph)[inter.units.edges,]))
    igraph::V(graph)[ igraph::V(graph)$name %in% v.to.preserve ]$preserve <- TRUE
  }
  
  if(conserve.fragments.balance){
    # the 'fragments balance' is computed here as it is computed by the frag.get.parameters() function:
    frag.balance.unit.1 <- .fragments.balance(graph)
    frag.balance.unit.2 <- 1 - frag.balance.unit.1

    n.frag.to.remove.unit.1 <- floor(n.frag.to.remove * frag.balance.unit.1)
    n.frag.to.remove.unit.2 <- floor(n.frag.to.remove * frag.balance.unit.2)
    
    # subgraphs
    unit.1.g <- igraph::induced_subgraph(graph, igraph::V(graph)[ v1 ] )
    unit.2.g <- igraph::induced_subgraph(graph, igraph::V(graph)[ v2 ] )
    
    final.frag.nr.unit.1 <- igraph::gorder(unit.1.g) - n.frag.to.remove.unit.1
    final.frag.nr.unit.2 <- igraph::gorder(unit.2.g) - n.frag.to.remove.unit.2

    if(final.frag.nr.unit.1 < 2) {final.frag.nr.unit.1 <- 2}
    if(final.frag.nr.unit.2 < 2) {final.frag.nr.unit.2 <- 2}
    
    # reduce subgraphs:
    unit.1.g <- .reduce.graph(unit.1.g, final.frag.nr = final.frag.nr.unit.1, conserve.objects.nr = conserve.objects.nr)
    unit.2.g <- .reduce.graph(unit.2.g, final.frag.nr = final.frag.nr.unit.2, conserve.objects.nr = conserve.objects.nr)
    
    # recombine subgraphs:
    igraph::V(unit.1.g)$name <- as.character(igraph::V(unit.1.g)$name)
    igraph::V(unit.2.g)$name <- as.character(igraph::V(unit.2.g)$name)
    graph.recombined <- igraph::union(unit.1.g, unit.2.g, byname = TRUE)
    
    # add inter-units edges:
    e.list <- igraph::as_edgelist(graph)[inter.units.edges, ]
    e.list <- e.list[e.list[, 1] %in% igraph::V(graph.recombined)$name & e.list[, 2] %in% igraph::V(graph.recombined)$name, ]
    e.list <- matrix(e.list, ncol = 2)
    
    if(nrow(e.list) > 0){
      graph <- igraph::add_edges(graph.recombined, as.character(c(rbind(e.list[, 1], e.list[, 2]))))
    } else {
      graph <- graph.recombined
    }
    
    # reformat attributes:
    igraph::V(graph)$layer <- .concatenate.attr(graph, "layer")
    igraph::V(graph)$morphometry <- .concatenate.attr(graph, "morphometry")
    
    graph <- Reduce(igraph::delete_vertex_attr, c("morphometry_1", "morphometry_2",
                                                  "object.id_1", "object.id_2",
                                                  "layer_1", "layer_2",
                                                  "preserve_1", "preserve_2"), 
                     graph)
    
    graph <- Reduce(igraph::delete_edge_attr, c("weight_1", "weight_2",
                                                 "scope_1", "scope_2",
                                                 "id_1", "id_2"), 
                     graph)
    graph$frag_type <- graph$frag_type_1
    graph <- Reduce(igraph::delete_graph_attr, c("frag_type_1", "frag_type_2"), graph)
      
  } else { # conserve.fragments.balance = FALSE
    final.frag.nr <- igraph::gorder(graph) - n.frag.to.remove
    graph.reduced <- .reduce.graph(graph, final.frag.nr, conserve.objects.nr)
    
    # add inter-units edges:
    e.list <- igraph::as_edgelist(graph)[inter.units.edges, ]
    e.list <- e.list[e.list[, 1] %in% igraph::V(graph.reduced)$name & e.list[, 2] %in% igraph::V(graph.reduced)$name, ]
    e.list <- matrix(e.list, ncol = 2)
    
    if(nrow(e.list) > 0){
      graph <- igraph::add_edges(graph.reduced, as.character(c(rbind(e.list[, 1], e.list[, 2]))))
    } else {
      graph <- graph.reduced
    }
    
  }
  graph <- igraph::simplify(graph)
  igraph::E(graph)$weight <- 1 # reset weight values
  igraph::delete_vertices(graph, igraph::V(graph)[igraph::degree(graph) == 0])
}

test_that("preserve component number (conserving objects nr)", {
  set.seed(1)
  g1 <- frag.simul.process(1, 10, 41)
  g2 <- frag.graph.reduce(g1, n.frag.to.remove =30, conserve.objects.nr = TRUE)
  g3 <- frag.graph.reduce(g1, n.frag.to.remove = 40, conserve.objects.nr = TRUE)

  expect_identical(components(g1)$no, expected = components(g2)$no)
  expect_identical(components(g1)$no, expected = components(g3)$no)
})



test_that("reduce vertice number (conserving objects nr)", {
  set.seed(1)
  g1 <- frag.simul.process(1, 10, 41)
  g2 <- frag.graph.reduce(g1, n.frag.to.remove = 30, conserve.objects.nr = TRUE)
  
  expect_identical(gorder(g2), 20)
})


test_that("reduce vertice number (not conserving objects nr)", {
  set.seed(1)
  g1 <- frag.simul.process(initial.layers=2, n.components = 10, vertices = 41, disturbance = 0.1)
  g2 <- frag.graph.reduce(g1, n.frag.to.remove = 30, conserve.objects.nr = FALSE, conserve.fragments.balance = FALSE, conserve.inter.units.connection = FALSE)
  
  expect_identical(gorder(g2), 11)
})


test_that("reduce vertice number (conserving objects nr, conserving fragments balance)", {
  set.seed(1)
  g1 <- frag.simul.process(initial.layers=2, n.components = 10, vertices = 51, disturbance = 0.15)
  g2 <- frag.graph.reduce(g1, n.frag.to.remove = 20, conserve.objects.nr = TRUE, conserve.fragments.balance = TRUE, conserve.inter.units.connection = FALSE)

  expect_equal(frag.get.parameters(g1, "layer")$balance,
               frag.get.parameters(g2, "layer")$balance,
               tolerance = 0.1)
    
  expect_equal(gorder(g2), 31)
  expect_lt(frag.relations.by.layers(g2, "layer")[2, 1],
            frag.relations.by.layers(g1,  "layer")[2, 1])
})


test_that("reduce vertice number (conserving fragments balance and inter-units connection)", {
  set.seed(1)
  g1 <- frag.simul.process(initial.layers=2, n.components = 11, vertices = 61, disturbance = 0.15)
  g2 <- frag.graph.reduce(g1, n.frag.to.remove = 30, conserve.objects.nr = FALSE, conserve.fragments.balance = TRUE, conserve.inter.units.connection = TRUE)
  
  expect_equal(frag.get.parameters(g1, "layer")$balance,
               frag.get.parameters(g2, "layer")$balance,
               tolerance = 0.08)
  
  expect_equal(gorder(g2), 32)
  expect_equal(frag.relations.by.layers(g2,  "layer")[2, 1], 14)
})

#
frag.diameters <- function(graph, cumulative=FALSE){
  # tests:
  .check.frag.graph(graph)
  
  if( ! is.null(igraph::graph_attr(graph, "frag_type")) ) {
    if(igraph::graph_attr(graph, "frag_type") == "connection and similarity"){
      warning("Diameter distribution is meaningless for 'connection and similarity' fragmentation graphs.")
    }
  }
  if(! is.logical(cumulative)) stop("The 'cumulative' parameter must be logical.")
  
  # main body:
  g.list <- igraph::decompose(graph)
  diameter.v <- sapply(g.list, igraph::diameter, weights=NA, directed=FALSE, unconnected=FALSE)
  diameter.v <- graphics::hist(diameter.v, breaks=0:max(diameter.v), plot=FALSE)$count 
  if(cumulative){
    diameter.v <- rev(cumsum(rev(diameter.v))) / sum(diameter.v)
  }
  names(diameter.v) <- 1:length(diameter.v)
  diameter.v
}

test_that("frag.diameters", {
  data(LiangAbu)
  abu.frag <- make_frag_object(cr=liangabu.connection, fragments=liangabu.fragments)
  abu.g <- make_cr_graph(abu.frag)
  expect_equal(as.integer(frag.diameters(abu.g)), c(24,  4,  0,  1,  1))
})

test_that("frag.diameters (with cumulative)", {
  data(LiangAbu)
  abu.frag <- make_frag_object(cr=liangabu.connection, fragments=liangabu.fragments)
  abu.g <- make_cr_graph(abu.frag)
  expect_equal(as.numeric(frag.diameters(abu.g, cumulative=TRUE)),
               c(1, 0.2, 0.06666667, 0.06666667, 0.03333333), tolerance = 0.00001)
})
  
  

.run.simul <- function(iter, observed.graph, layer.attr, initial.layer, verbose, ...){
  res <- lapply(seq_len(iter), function(i){
    g <- frag.simul.process(initial.layers = initial.layer,
                            from.observed.graph = observed.graph,
                            observed.layer.attr = layer.attr,  ...)
    # measure the properties of the graph:
    params <- frag.get.parameters(g, "layer", verbose = verbose)
    c(
      "edges" = params$edges,
      "balance" = params$balance,
      "components.balance" = params$components.balance,
      "disturbance" = params$disturbance,
      frag.layers.admixture(g, "layer", verbose = verbose),
      "cohesion" = rbind(frag.layers.cohesion(g, "layer", verbose = verbose)),
      "edge.weights.sum" = sum(igraph::E(g)$weight),
      "edge.weights.median" = params$edge.weights.median,
      "edge.weights.median.abs.dev." = params$edge.weights.median.abs.dev.
    )
  })
  res <- do.call(rbind, res)
  data.frame(res)
}  

frag.simul.compare <- function(graph, layer.attr, iter, summarise=TRUE, verbose=TRUE, ...){
  # tests:
  .check.frag.graph(graph)
  .check.layer.argument(graph, layer.attr)
  
  if(iter < 30) stop("At least 30 iterations are required.")
  if(! is.logical(summarise)){
    stop("A logical value is required for the 'summary' parameter.")
  }
  # main function:
  resH1 <- .run.simul(iter, graph, layer.attr, initial.layer = 1, verbose = verbose, ...)
  resH2 <- .run.simul(iter, graph, layer.attr, initial.layer = 2, verbose = verbose, ...)
  
  if(! summarise){
    return(list("h1.data" = resH1, "h2.data" = resH2))
  }
  stats <- frag.simul.summarise(graph, layer.attr, resH1, resH2, verbose = verbose)
  # print the summary table:
  print(stats)
  #return silently the results as a list:
  invisible(list("h1.data" = resH1, "h2.data" = resH2, "summary" = stats))
}

test_that("frag.simul.compare works", {
  set.seed(1)
  g <- frag.simul.process(n.components=20, vertices=50, disturbance=.15, verbose=FALSE)
  res <- frag.simul.compare(g, layer.attr="layer", iter=60, verbose=FALSE)
  expect_equal(nrow(res[[1]]), 60)
  expect_equal(nrow(res[[2]]), 60)
  
  if (isNamespaceLoaded("RBGL")) { #  due to issues with RBGL on Win-build
    expect_equal(res[[3]]$`Obs. value/H1`, c("lower", "within", "within", "within", "within", "within", "within", "lower", "lower", "lower"))
    expect_equal(res[[3]]$`p.value`, c("0.4",  "0.38", "0.39", "0.36", "0.08", "0.01", "0", "0.57", "0.25", "0.24"))
  } else {
    expect_equal(res[[3]]$`Obs. value/H1`, c("lower", "within", "within", "within", "within", "within", "within", "lower", "lower", "lower"))
    expect_equal(res[[3]]$`p.value`, c("0.45",  "0.38", "0.39", "0.36", "0.09", "0.02", "0", "0.55", "0.25", "0.23"))
  }
})

test_that("parameters of a constrained generated graph with 1 initial layer are correct", {
  set.seed(1)
  g <- frag.simul.process(n.components=20, vertices=50, disturbance=.15, verbose=FALSE)
  res <- frag.simul.compare(g, layer.attr="layer", iter=60, summarise=FALSE, verbose=FALSE)
  res <- frag.simul.summarise(g, layer.attr="layer", res.h1=res[[1]], res.h2=res[[2]], verbose=FALSE)
  expect_equal(as.character(res$`Obs. value/H1`), 
               c("lower", "within", "within", "within", "within", "within", "within", "lower",  "lower", "lower"))
})

.subsetsum <- function(x, target, i = 1){ # TODO: add comments to this function
  while(i != length(x)){
    s <- sum(x[seq_len(i)], na.rm = TRUE)
    if(s == target) break
    if(s > target) x[i] <- NA
    i <- i + 1
  }
  x[seq(i+1, length(x))] <- NA
  which(! is.na(x))
}

.select.component <- function(g, aggreg.factor){
  # high aggreg.factor favors bigger components:
  proba <- 1/ (1 + (seq_len(igraph::components(g)$no) * aggreg.factor))
  sample(order(igraph::components(g)$csize, decreasing = T), 1, prob = proba)
}

.connect.neighbors <- function(g, v, v.to.add.name, edges, planar){
  # if there are other neighbours than the new vertex, try to connect one of them with the new vertex.
  # identify the neighbors of the target vertex:
  neighbors <- igraph::ego(g, order = 1, nodes = v, mindist = 1)
  neighbors <- unlist(neighbors)
  neighbors <- neighbors[ ! neighbors == v.to.add.name]
  neighbors <- as.character(neighbors)
  # select none or one of the neighbors:
  neighbors <- sample(neighbors, sample(0:length(neighbors), 1))
  if(length(neighbors) > 0 & (igraph::gsize(g) + 1) < edges ){
    if(planar){
      # make a temporary version of the modified component of the graph:
      g.tmp <- igraph::induced_subgraph(g, igraph::V(g)[ igraph::V(g)$object.id == igraph::V(g)[v]$object.id ])
      g.tmp <- igraph::add_edges(g.tmp, as.character(c(rbind(v.to.add.name, neighbors)) ))
      # if the modified component is still planar, then connect the new vertex and the neighbor: 
      if(RBGL::boyerMyrvoldPlanarityTest(igraph::as_graphnel(g.tmp))){
        g <- igraph::add_edges(g, as.character(c(rbind(v.to.add.name, neighbors)) ))
      }
    }else{ # if no planarity constraint:
      g <- igraph::add_edges(g, c(rbind(v.to.add.name, neighbors)) )
    }
  }
  g
}

.add.fragment <- function(g, n.components, edges, connect.neighbors, planar, aggreg.factor){
  # select component:
  compo.to.fragment <- .select.component(g, aggreg.factor)
  
  v.to.fragment <- igraph::V(g)[ igraph::V(g)$object.id == compo.to.fragment]$name
  # select the target vertex in the component:
  v <-  sample(as.character(v.to.fragment), 1)
  # add a new vertex and an edge with the target vertex:
  v.to.add.name <- igraph::gorder(g) + 1
  g <- igraph::add_vertices(g, 1, attr = list("name" = v.to.add.name, 
                                      "object.id" = igraph::V(g)[v]$object.id ))
  g <- igraph::add_edges(g, c(v, v.to.add.name))
  
  # try to connect the new vertex and the neighbors of the target vertex:
  if(connect.neighbors){
    g <- .connect.neighbors(g, v, v.to.add.name, edges, planar)
  }
  g
}




.main <- function(n.components, vertices, edges, balance, disturbance, aggreg.factor, planar){
  stop.msg <- "No solution with these parameters, decrease the number of 'edges', increase the number of 'vertices', or remove one of these constraints."
  # Initialize graph:  
  
  g <- n.components * igraph::make_graph(c(1, 2), directed = FALSE)
  igraph::V(g)$name <- seq_len(igraph::gorder(g))
  igraph::V(g)$object.id <- igraph::components(g)$membership
  # Build graph:
  if(is.infinite(vertices)){ # only edge count constraint
    if(vertices/2 < n.components){
      stop(stop.msg)}
    while(igraph::gsize(g) < edges){
      g <- .add.fragment(g, n.components, edges, connect.neighbors=T, planar, aggreg.factor)
    }
  } else if(is.infinite(edges)){ # only vertices count constraint
    if(edges < n.components){
      stop("Increase 'edges' or decrease 'n.components'.")}
    while(igraph::gorder(g) < vertices){
      g <- .add.fragment(g, n.components, edges, connect.neighbors=T, planar, aggreg.factor)
    }
  } else if(! is.infinite(vertices) & ! is.infinite(edges) ){ # vertices & edges constraints
    if(edges < n.components + (vertices - 2 * n.components) ){
      stop(stop.msg) 
    }   # if not enough edges to add vertices:
    if(vertices - n.components * 2 >  edges - n.components ){
      stop(stop.msg)
    }
    # if(edges > 3 * (vertices - (n.components - 1)*2) - 7 + n.components){ # uncorrect, don't work for graphs only made of dyads
    if(edges > 3 * (vertices - (n.components - 1)*2) - 6 + n.components){ # adapted version, but probably uncorrect
      stop(stop.msg)
    }
    
    while(igraph::gorder(g) < vertices & igraph::gsize(g) < edges ){
      g <- .add.fragment(g, n.components, edges, connect.neighbors=F, planar, aggreg.factor)
    }
    # check if the number of supplementary edges conserve planarity:
    e.existing <- 0 # initialize the value
    e.max <- edges  # initialize the value
    components.size <- igraph::components(g)$csize
    gsub <- igraph::induced_subgraph(g,
                             igraph::V(g)[ igraph::components(g)$membership %in% which(components.size > 2)])
    gsub <- igraph::decompose(gsub)
    if(length(gsub) > 0){
      e.existing <- sapply(gsub, igraph::gsize)
      e.max <- sapply(components.size[components.size > 2], function(x)  3*x-6) # e max for planar graphs
      e.max <- sum(e.max - e.existing)
    }
    
    if(edges - igraph::gsize(g) > e.max){ # stop if the graph already contains too many edges
      stop(stop.msg)
    }
    while(igraph::gsize(g) < edges){
      # select a component:
      selected.component <- .select.component(g, aggreg.factor)
      v.to.connect <- igraph::V(g)[ igraph::V(g)$object.id == selected.component]$name
      # select two vertices in the component:
      v.to.connect <-  sample(as.character(v.to.connect), 2)
      if(length(igraph::E(g)[v.to.connect[1] %--% v.to.connect[2]]) == 0){# if the edge does not exist yet
        # check planarity of the component before adding edge:
        if(planar){
          g.tmp <- igraph::induced_subgraph(g, igraph::V(g)[ igraph::V(g)$object.id == selected.component])
          g.tmp <- igraph::add_edges(g, v.to.connect)
          
          if(RBGL::boyerMyrvoldPlanarityTest(igraph::as_graphnel(g.tmp))){
            g <- igraph::add_edges(g, v.to.connect)
          }
        }else{
          g <- igraph::add_edges(g, v.to.connect)
        }
      }
    } # end of the while loop
  } # end of the elseif "vertices + edges constraints"
  g
}


.add.disturbance <- function(g, nr.v.to.disturb, asymmetric.transport.from){
  # default behaviour:
  v.to.disturb <- sample(seq(1, igraph::gorder(g)), nr.v.to.disturb)
  # if asymmetric.transport.from is set:
  if(asymmetric.transport.from != 0){
    if(nr.v.to.disturb <= length(igraph::V(g)[ igraph::V(g)$layer == asymmetric.transport.from]) ){
      v.to.disturb <- sample(igraph::V(g)[ igraph::V(g)$layer == asymmetric.transport.from], nr.v.to.disturb)
    } else{
      stop("The number of fragments for asymmetric transport exceeds the number of fragments in this layer. Decrease the 'disturbance' value.")
    }
  } 
  # reverse layer values of the selected vertices:
  igraph::V(g)[v.to.disturb]$layer <- as.character(factor(igraph::V(g)[v.to.disturb]$layer,
                                                  levels = c(1,2), labels = c(2,1)))
  g
}


 

frag.simul.process <- function(initial.layers=2, n.components=NULL, vertices=Inf, edges=Inf, balance=.5, components.balance=.5, disturbance=0, aggreg.factor=0, planar=FALSE, asymmetric.transport.from=NULL, from.observed.graph=NULL, observed.layer.attr=NULL, verbose=TRUE){
  
  if(! is.logical(planar)) stop("The 'planar' argument must be logical.")
  if(planar==TRUE & (! requireNamespace("RBGL", quietly=TRUE))){
    stop("To use the `planar` constraint, the RBGL package is required.")
  }
  
  #  If required by the user, use parameters from the observed graph (except the number of edges):
  if( ! is.null(from.observed.graph)){
    if ( is.null(observed.layer.attr) ) stop("The 'observed.layer.attr' parameter is missing.")
    if( ! is.character(observed.layer.attr))  stop("The parameter 'observed.layer.attr' requires a character value.")
    if( ! observed.layer.attr %in% names(igraph::vertex_attr(from.observed.graph)) ){
      stop(paste("No '", observed.layer.attr, "' vertices attribute.", sep=""))
    }
    if(length(unique(igraph::vertex_attr(from.observed.graph, observed.layer.attr))) != 2){
      stop("The `layer` attribute of the observed graph must contain two layers.")
    }
    # retrieve the observed graph's values:
    params <- frag.get.parameters(from.observed.graph, observed.layer.attr, verbose = verbose) 
    # set the parameters with observed values if not already set by the user:
    if(missing(n.components)) n.components <- params$n.components
    if(missing(vertices)) vertices <- params$vertices
    if(missing(balance)) balance <- params$balance
    if(missing(components.balance)) components.balance <- params$components.balance
    if(missing(disturbance)) disturbance <- params$disturbance
    if(missing(aggreg.factor)) aggreg.factor <- params$aggreg.factor
    if(missing(planar))  planar <- params$planar
    if(is.na(planar)) {
      planar <- FALSE
      if(verbose) message("The planarity of the graph value is indeterminated, simulations are executed with no planar constraint.")
    }
  }
  # BEGIN Tests:
  if(is.null(n.components)) stop("The 'n.components' parameter is required.")

  if(! is.numeric(balance)){
    stop("The 'balance' argument requires a numerical value.")
  } else if(balance <= 0 | balance >= 1){
    stop("'balance' values must range in ]0;1[")
  }

  if(! is.numeric(components.balance)){
    stop("The 'components.balance' argument requires a numerical value.")
  } else if(components.balance <= 0 | components.balance >= 1){
    stop("'components.balance' values must range in ]0;1[")
  }

  if(! is.numeric(disturbance)){
    stop("The 'disturbance' argument requires a numerical value.")
  } else if(disturbance < 0 | disturbance > 1){
    stop("'disturbance' values must range in [0;1].")
  }

  if(is.infinite(vertices) & is.infinite(edges)){
    stop("At least one of the parameters 'vertices' or 'edges' is required.")
  }
  if(is.infinite(vertices) & initial.layers == 2 ){
    stop("With 2 initial layers, the vertices parameter is required.")
  }
  
  if(! initial.layers %in% c(1, 2)){
    stop("The 'initial.layers' parameter requires a numerical value of 1 or 2.")
  }

  if(! is.numeric(aggreg.factor)){
    stop("The 'disturbance' argument requires a numerical value.")
  } else if(aggreg.factor > 1 | aggreg.factor < 0 ){
    stop("The 'aggreg.factor' parameter must range in [0;1].")
  }

  if(missing(asymmetric.transport.from) | is.null(asymmetric.transport.from)) asymmetric.transport.from <- 0
  if(! asymmetric.transport.from %in% c(0, 1, 2, "0" ,"1", "2")){
    stop("Values for 'asymmetric.transport.from' must be one of 1, 2 or 0.")
  }
  
  if(n.components > vertices / 2){
      stop("'n.components' must be <= 'vertices' / 2.")
  }  
  # END tests
  
  # BEGIN main body of the function:
  
  if(initial.layers == 1){ # for 1 initial layer ----
    g <- .main(n.components, vertices, edges, balance, disturbance, aggreg.factor, planar) 
    
    # BALANCE. Determine layer size:
    v.layer1 <- round(igraph::gorder(g) * balance)
    
    # search possible combinations of components and use the first one:
    sel.components <- igraph::components(g)$csize
    names(sel.components) <- seq_len(length(sel.components))
    sel.components <- .subsetsum(sample(sel.components), v.layer1) # randomize order
    sel.components <- names(sel.components)
    
    # assign layers:
    igraph::V(g)$layer <- 2
    igraph::V(g)[ igraph::V(g)$object.id %in% sel.components ]$layer <- 1
    
    # ADD DISTURBANCE:
    nr.v.to.disturb <- round(igraph::gorder(g) * disturbance)
    if(nr.v.to.disturb > 0){
      g <- .add.disturbance(g, nr.v.to.disturb, asymmetric.transport.from)
    }
  }
  
  if(initial.layers == 2){ # for 2 initial layers ----
    if(! is.infinite(edges)){
      warning("With two initial layers, the 'edge' parameter is not used.")
    }
    n.components.l1 <- round(n.components * components.balance)
    n.components.l2 <- n.components - n.components.l1
    if(n.components.l1 == 0 | n.components.l2 == 0){
      stop("The 'components.balance' is too low or too high.")
    }
    vertices.l1 <- round(vertices * balance)
    vertices.l2 <- vertices - vertices.l1
    
    g.layer1 <- .main(n.components.l1,
                      vertices.l1,  
                      edges = Inf,  # TODO: répartir le nombre d'arêtes avec la proportion de frag.balance 
                      balance = .5,
                      disturbance = 0,
                      aggreg.factor,
                      planar)
    g.layer2 <- .main(n.components.l2,
                      vertices.l2,
                      edges = Inf,
                      balance = .5,
                      disturbance = 0,
                      aggreg.factor,
                      planar) 
    # tag and merge the two graphs:
    igraph::V(g.layer1)$layer <- 1
    igraph::V(g.layer2)$layer <- 2
    igraph::V(g.layer1)$name <- as.character(igraph::V(g.layer1)$name)
    igraph::V(g.layer2)$name <- paste(igraph::V(g.layer2)$name, ".2", sep="")
    g <- igraph::disjoint_union(g.layer1, g.layer2)
    # ADD DISTURBANCE:
    nr.v.to.disturb <- round(igraph::gorder(g) * disturbance)
    if(nr.v.to.disturb > 0){
      g <- .add.disturbance(g, nr.v.to.disturb, asymmetric.transport.from)
    }
  }
  # finalize and return the graph:
  g <- frag.edges.weighting(g, layer.attr="layer")
  g <- igraph::delete_vertex_attr(g, "which")
  g$frag_type <- "cr"
  g
}

set.seed(1)
g1 <- frag.simul.process(n.components=5,  vertices=20, disturbance=.1)
g2 <- frag.simul.process(n.components=10, vertices=90, disturbance=.03)
g3 <- frag.simul.process(n.components=20, vertices=50, disturbance=.4)

igraph::V(g2)$name <- paste0(igraph::V(g2)$name, "g2")
igraph::V(g3)$name <- paste0(igraph::V(g3)$name, "g3") 

igraph::V(g2)$layer <- as.character(factor(igraph::V(g2)$layer, labels = c(3, 4))) 
igraph::V(g3)$layer <- as.character(factor(igraph::V(g3)$layer, labels = c(5, 6))) 

g1234 <- igraph::disjoint_union(g1, g2) 
g1234$frag_type <- "cr" 

g1256 <- igraph::disjoint_union(g1, g3) 
g1256$frag_type <- "cr" 

cohesion.1234 <- frag.layers.cohesion(g1234, layer.attr="layer", verbose=FALSE) 
cohesion.1256 <- frag.layers.cohesion(g1256, layer.attr="layer", verbose=FALSE) 


test_that("add.math.signs works", {
  rank <- frag.cohesion.ranking(cohesion.1234, add.math.signs = TRUE) 
  
  expect_equal(names(rank), c("3", ">", "4", ">", "2", ">", "1", ""))
})



test_that("labels rank and count are correct in case 1", {
  rank <- frag.cohesion.ranking(cohesion.1234, add.math.signs = FALSE) 
  
  expect_equal(names(rank), c("3", "4", "2", "1"))
  expect_equal(c(rank, use.names=FALSE), c(3, 2, 1, 0))
})

test_that("labels ranks are different in case 2", {
  rank <- frag.cohesion.ranking(cohesion.1256, add.math.signs = FALSE) 
  
  expect_equal(names(rank), c("2", "1", "6", "5"))
  expect_equal(c(rank, use.names=FALSE), c(3, 2, 1, 0))
})

# ONE INITIAL LAYER ####

test_that("parameters of a constrained generated graph with 1 initial layer are correct", {
  set.seed(1)
  g <- frag.simul.process(initial.layers=1, 
                          n.components=22,
                          vertices=60,
                          edges=55,
                          disturbance=0.1)
  expect_equal(igraph::gsize(g), 55)
  expect_equal(igraph::gorder(g), 60)
  expect_equal(igraph::components(g)$no, 22)
})


test_that("number of not constrained vertices is correct", {
  set.seed(1)
  g <- frag.simul.process(initial.layers=1, 
                          n.components=22,
                          edges=55,
                          disturbance=0.1)
  expect_equal(igraph::gorder(g), 67)
})

test_that("cohesion values of a graph with 1 initial layer are correct", {
  set.seed(1)
  g <- frag.simul.process(initial.layers=1, 
                          n.components=20,
                          vertices=55,
                          edges=45,
                          disturbance=0.1)
  expect_equal(frag.layers.cohesion(g, "layer")[1], 0.5123, tolerance=.00001)
  expect_equal(frag.layers.cohesion(g, "layer")[2], 0.4181, tolerance=.00001)
  expect_equal(as.numeric(frag.layers.admixture(g, "layer")), 0.0696)
})



# TWO INITIAL LAYERS ####

test_that("parameters of a generated graph with 2 initial layers are correct", {
  set.seed(1)
  g <- frag.simul.process(initial.layers=2, 
                          n.components=20,
                          vertices=62,
                          disturbance=0.1)
  expect_equal(igraph::gsize(g), 55)
  expect_equal(igraph::gorder(g), 62)
  expect_equal(igraph::components(g)$no, 20)
})


test_that("admixture and cohesion values of a graph with 2 initial layers are correct", {
  set.seed(1)
  g <- frag.simul.process(initial.layers=2, 
                          n.components=21,
                          vertices=55,
                          disturbance=0.15)
  expect_equal(frag.layers.cohesion(g, "layer")[1], 0.4910982, tolerance=.00001)
  expect_equal(frag.layers.cohesion(g, "layer")[2], 0.4151, tolerance=.00001)
  expect_equal(as.numeric(frag.layers.admixture(g, "layer")), 0.0938, tolerance=.00001)
})

test_that("the optional asymmetric.transport.from parameter works", {
  set.seed(1)
  g1 <- frag.simul.process(initial.layers=2, 
                          n.components=21,
                          vertices=55,
                          disturbance=0.15,
                          asymmetric.transport.from=1)
  g2 <- frag.simul.process(initial.layers=2, 
                          n.components=21,
                          vertices=55,
                          disturbance=0.15,
                          asymmetric.transport.from=2)
  expect_equal(as.numeric(table(igraph::V(g1)$layer)[1]), 20)
  expect_equal(as.numeric(table(igraph::V(g2)$layer)[1]), 36)
})


test_that("the optional from.observed.graph parameter works", {
  set.seed(1)
  g1 <- frag.simul.process(initial.layers=2, 
                           n.components=21,
                           vertices=55,
                           disturbance=0.15,
                           asymmetric.transport.from=1)
  g2 <- frag.simul.process(from.observed.graph=g1, observed.layer.attr="layer")
  expect_equal(class(g2), "igraph")
  expect_equal(igraph::gorder(g1), igraph::gorder(g2))
})

.compare.values <- function(h1.values, h2.values, obs.value){
  obs.value <- unlist(obs.value)
  h1.values <- unlist(h1.values)
  h2.values <- unlist(h2.values)
  # difference between H1 and H2
  wilcox.res <- stats::wilcox.test(h1.values, h2.values, exact=FALSE)$p.value
  
  if(is.nan(wilcox.res)) return(c(NA,NA,NA,NA))
  
  # result for the wilcoxon test:
  ccl0 <- F
  if(wilcox.res < 0.05){
    ccl0 <- T
  }
  # observed value compared to H1 values:
  if(obs.value < summary(h1.values)[2]){
    ccl1 <- "lower"
  } else if(obs.value > summary(h1.values)[5]){
    ccl1 <- "higher"
  } else{
    ccl1 <- "within"
  }
  # observed value compared to H2 values:
  if(obs.value < summary(h2.values)[2]){
    ccl2 <- "lower"
  } else if(obs.value > summary(h2.values)[5]){
    ccl2 <- "higher"
  } else{
    ccl2 <- "within"
  }
  c("H1 != H2?" = ccl0, "p.value" = round(wilcox.res, 2),
    "Obs. value/H1" = ccl1, "Obs. value/H2" = ccl2)
}

frag.simul.summarise <- function(graph, layer.attr, res.h1, res.h2,
                                 cohesion1.attr = "cohesion1",
                                 cohesion2.attr = "cohesion2",
                                 admixture.attr = "admixture",
                                 verbose = TRUE){
  # todo: add params:  
  # tests:
  .check.frag.graph(graph)
  .check.layer.argument(graph, layer.attr)
  
  if(is.null(res.h1) | is.null(res.h2)){
    stop("res.h1 and res.h2 are mandatory.")
  }
  if(sum(! colnames(res.h1) == colnames(res.h2)) != 0){
    stop("The column names of res.h1 and res.h2 are not identical.")
  }
  if(! (is.data.frame(res.h1) | is.data.frame(res.h2)) ){
    stop("Data frames are required for the res.h1 and res.h2 parameters.")
  }
  if( ! (is.character(cohesion1.attr) | is.character(cohesion2.attr) | is.character(admixture.attr))){
    stop("Character strings are required for the cohesion1.attr, cohesion2.attr, and admixture parameters.")
  }
  if( sum(c(cohesion1.attr, cohesion2.attr, admixture.attr) %in% colnames(res.h1))  != 3 ){
    stop(paste("No column named '", cohesion1.attr, "', '", cohesion2.attr, "' or '", admixture.attr, "' in the data frames.", sep=""))
  }
  colnames(res.h1)[which(colnames(res.h1) == cohesion1.attr)] <- "cohesion1"
  colnames(res.h1)[which(colnames(res.h1) == cohesion2.attr)] <- "cohesion2"
  colnames(res.h1)[which(colnames(res.h1) == admixture.attr)] <- "admixture"
  colnames(res.h2)[which(colnames(res.h2) == cohesion1.attr)] <- "cohesion1"
  colnames(res.h2)[which(colnames(res.h2) == cohesion2.attr)] <- "cohesion2"
  colnames(res.h2)[which(colnames(res.h2) == admixture.attr)] <- "admixture"
  
  # retrieve the parameters of the observed graph:  
  obs.params <- c(frag.get.parameters(graph, layer.attr, verbose = verbose),
                  frag.layers.admixture(graph, layer.attr, verbose = verbose),
                  "cohesion" = frag.layers.cohesion(graph, layer.attr, verbose = verbose),
                  "edge.weights.sum" = sum(igraph::E(graph)$weight))
  
  if(verbose & sum(! colnames(res.h1) %in% names(obs.params)) != 0){
    warning("Some simulated parameters are missing in the observed graph.")
  }
  # parameters in the observed graph and the simulated results:
  params <- intersect(colnames(res.h1), names(obs.params))
  
  # compare the observed and simulated parameters:
  res <- sapply(params, function(param)
    .compare.values(res.h1[param], res.h2[param], obs.params[param]))
  # results:
  as.data.frame(t(res))
}

frag.path.lengths <- function(graph, cumulative=FALSE){
  # tests:
  .check.frag.graph(graph)
  # main function:  
  if(! is.logical(cumulative)) stop("The 'cumulative' parameter requires a logical value.")
  
  path.vector <- igraph::distance_table(graph, directed = FALSE)$res
  
  if(cumulative){
    path.vector <- path.vector / max(path.vector)
  }
  path.vector
}

frag.cohesion.ranking <- function(cohesion.df, add.math.signs = TRUE){
  if(nrow(cohesion.df) == 1) return()
  cohesion.df <- data.frame(cohesion.df)
  
  cohesion.df$unit1 <- gsub("^(.*?) ?/ ?(.*?)$", "\\1", rownames(cohesion.df), perl = TRUE)
  cohesion.df$unit2 <- gsub("^(.*?) ?/ ?(.*?)$", "\\2", rownames(cohesion.df), perl = TRUE)
  
  sp.units <- unique(c(cohesion.df$unit1, cohesion.df$unit2))
  
  count <- apply(cohesion.df, 1, function(x) x[c(ncol(cohesion.df) -1, ncol(cohesion.df))][order(x[1:2])[2] ] )
  count <- sort(table(count), decreasing = TRUE)
  count <- t(as.data.frame(count, stringsAsFactors = FALSE))
  
  sp.units <- sp.units[ ! sp.units %in% count[1, ]]
  
  count <- cbind(count, rbind(sp.units, 0))
  res <- as.numeric(count[2, ])
  names(res) <- count[1, ]
  
  if( ! add.math.signs) return(res)
  
  operators <- sapply(seq_len(length(res) -1) ,  function(x)  res[x + 1] - res[x])
  operators.char <- operators
  operators.char[operators < 0 ] <- ">"
  operators.char[operators == 0 ] <- "="
  operators.char <- c(operators.char, "")
  
  res <- rbind(
    c(rbind(names(res), operators.char)),
    c(rbind(res, rep("", length(res))))
  )
  
  res <- data.frame(res)
  colnames(res) <- res[1,]
  rownames(res) <- c("", "Count")
  res[-1, ]
}

.randomly.delete.edges <- function(graph, value, remove.vertices){ 
  set.seed(10)  # reset the random value to ensure that the same series of edges is remove
  n.edges <- round(igraph::gsize(graph) * value)
  graph <- igraph::delete_edges(graph, sample(igraph::E(graph), n.edges))
  if(remove.vertices){
    graph <- igraph::delete_vertices(graph, igraph::degree(graph)==0)
  }
  graph
}

frag.observer.failure <- function(graph, likelihood, remove.vertices=FALSE){
  # output: a list of altered graphs with edges (and vertices) removed
  # tests:
  .check.frag.graph(graph)
  # main function:  
  if(! is.numeric(likelihood)){
    stop("The 'likelood' parameter requires a numerical value.")
  } else  if(sum(likelihood < 0, likelihood > 1)){
    stop("The 'likelihood' values must range in [0,1].")
  }
  # apply the subsection and return results:
  lapply(likelihood, function(value) .randomly.delete.edges(graph, value, remove.vertices))
}

.check.frag.graph <- function(graph){
  if(! igraph::is_igraph(graph)) stop("Not a graph object")
  if(igraph::is_directed(graph)) stop("The 'graph' parameter requires an undirected igraph object.")
  if(! igraph::is_simple(graph)) stop("Loops and multiple edges are not allowed.")
}

.check.layer.argument <- function(graph, layer.attr){
  if(is.null(igraph::vertex_attr(graph, layer.attr)))   stop("'layer.attr' is missing or does not correspond to a vertex attribute of the graph.")
  if(is.null(igraph::vertex_attr(graph, layer.attr)))   stop("The parameter 'layer.attr' is required.")
  if (is.null(layer.attr))  stop("No 'layer.attr' argument")
  if( ! is.character(layer.attr))  stop("The parameter 'layer.attr' requires a character value.")
  if( ! layer.attr %in% names(igraph::vertex_attr(graph)) ){
    stop(paste("There is no '", layer.attr, "' vertices attribute.", sep=""))
  }
}

# considering the subgraph including only fragments connected to fragments from the same spatial unit, proportion of fragments in the 1st spatial unit (alphanumerically)
.fragments.balance <- function(g){
  v1 <- igraph::V(g)[igraph::V(g)$layer == unique(igraph::V(g)$layer)[1]]
  v2 <- igraph::V(g)[igraph::V(g)$layer == unique(igraph::V(g)$layer)[2]]
  subgraph <- igraph::subgraph_from_edges(g, igraph::E(g)[ ! v1 %--% v2 ])
  frag.count <- table(igraph::V(subgraph)$layer)
  round(frag.count[1] / sum(frag.count), 2)
}


# considering the subgraph including only fragments connected to fragments from the same spatial unit, proportion of components in the 1st spatial unit (alphanumerically)
.components.balance <- function(g){
  v1 <- igraph::V(g)[igraph::V(g)$layer == unique(igraph::V(g)$layer)[1]]
  v2 <- igraph::V(g)[igraph::V(g)$layer == unique(igraph::V(g)$layer)[2]]
  subgraph <- igraph::subgraph_from_edges(g, igraph::E(g)[ ! v1 %--% v2 ])
  
  compo.balance <- sapply(igraph::decompose(subgraph), 
                          function(x) igraph::V(x)$layer[1])
  compo.balance <- table(compo.balance)
  round(compo.balance[1] / sum(compo.balance), 2) 
}

frag.get.layers <- function(graph, layer.attr, sel.layers){
  # tests:
  .check.frag.graph(graph)
  .check.layer.argument(graph, layer.attr)
  
  # function:
  layers <- igraph::vertex_attr(graph, layer.attr)
  if(sum(sel.layers %in% layers) != length(sel.layers)) stop("Some of the 'selected layers' are not in the 'layers' vector.")
  
  g.list <- lapply(sel.layers,
                   function(x) igraph::induced_subgraph(graph, V(graph)[layers == x] ))
  names(g.list) <- sel.layers
  g.list
}

#   Frag.object CLASS-SPECIFIC FUNCTIONS ####

setGeneric(
  name= "make_crsr_graph",
  def = function(object){standardGeneric("make_crsr_graph")}
)


setGeneric(
  name= "make_cr_graph",
  def = function(object){standardGeneric("make_cr_graph")}
)

setGeneric(
  name= "make_sr_graph",
  def = function(object){standardGeneric("make_sr_graph")}
)

make_frag_set_validation <- function(object)
{
  if( ncol(object@df.cr) == 1 & ncol(object@df.sr) == 1 ){
    stop("At least one of the 'cr' and 'sr' arguments is required.")
  }
  if(  object@frag_type %in% c("sr", "crsr")) {
    if( ! ncol(object@df.sr) > 1){
      stop("The data frame for the 'sr' argument must have at least two columns.")
    }
    if( sum( ! object@df.sr[,1] %in% object@fragments.df[, 1]) != 0){
      stop("Some objects in the similarity relationships data ('sr' parameter) are not documented in fragments data ('fragments' parameter)")
    }
  }  

  if( object@frag_type %in% c("cr", "crsr") ) {
    if( ! ncol(object@df.cr) > 1){
      stop("The data frame for the 'cr' argument must have at least two columns.")
    }
    if( sum( ! c(object@df.cr[,1], object@df.cr[,2]) %in% object@fragments.df[, 1]) != 0){
      stop("Some objects in the connection relationships data ('cr' parameter) are not documented in fragments data ('fragments' parameter)")
    }
  } 
 
  return(TRUE)
}

 


setClass(
  Class="Frag.object", 
  representation=representation(
    df.cr="matrix",
    df.sr="matrix",
    fragments.df="data.frame",
    frag_type="character"
  ),
  prototype = prototype(),
  validity = make_frag_set_validation
)

setMethod( # show  ----
  f="show",
  signature = "Frag.object",
  definition = function (object){
    message(paste(
      "------ Frag.object ------",
      "\n*   Frag_type = ", object@frag_type,
      "\n*   N fragments = ",
      length(unique( stats::na.omit(c(object@df.cr[,1],
                                      try(object@df.cr[,2], silent=TRUE),
                                      object@df.sr[,1]))) ),
      "\n-------------------------",
      sep=""))
  }
)

setMethod(  # make_cr_graph ----
  f = "make_cr_graph",
  signature = "Frag.object",
  definition = function(object)
  {
    if( object@frag_type == "sr" ){
      stop("No data available for connection relationships.")
    }
    cr.net <- igraph::graph_from_data_frame(object@df.cr, directed=FALSE,
                                            vertices=object@fragments.df)
    cr.net <- igraph::delete_vertices(cr.net, 
                                      igraph::degree(cr.net, mode="total") == 0)
    cr.net <- simplify(cr.net, remove.multiple = TRUE)
    igraph::E(cr.net)$type_relation <- "cr"
    cr.net <- igraph::set_graph_attr(cr.net, "frag_type", "connection relations")
    return(cr.net)
  }
)

setMethod(  # make_sr_graph ----
  f = "make_sr_graph", 
  signature = "Frag.object",
  definition = function(object)
  {
    
    if( object@frag_type == "cr" ){
      stop("No data available for similarity relationships.")
    }
    #  'similarity units' ids are recoded to avoid confusion with the fragments ids:
    object@df.sr[,2] <- as.character(factor(object@df.sr[,2], 
      labels = paste("su", c(1:(0 + length(unique((object@df.sr[,2]))) )))
    ) )
    vertices.list <- unique( c(object@df.sr[, 1], object@df.sr[, 2]) ) 
    sr.net <- igraph::simplify(igraph::graph_from_data_frame(object@df.sr[, 1:2], 
                                                 directed=TRUE, vertices=vertices.list ))
    sr.net <- igraph::graph_from_adjacency_matrix(igraph::bibcoupling(sr.net),
                                                  diag=FALSE, mode="undirected" )
    sr.net <- igraph::delete_vertices(sr.net, igraph::degree(sr.net, mode="total") == 0)
    # vertices attributes:
    fragments.df <- object@fragments.df
    names(fragments.df)[1] <- "name"
    
    attributes <- merge(       #retrieve the  graph attributes
      cbind("name" = igraph::V(sr.net)$name),
      cbind(fragments.df),
      by="name", sort=FALSE)
    igraph::vertex_attr(sr.net) <- lapply(attributes, as.character) #add vertex attributes
    
    # clean graph:
    sr.net <- igraph::delete_vertices(sr.net, igraph::degree(sr.net) == 0)
    # Set attributes:
    igraph::E(sr.net)$type_relation <- "sr"
    sr.net <- igraph::set_graph_attr(sr.net, "frag_type", "similarity relations")
    return(sr.net)
  }
)

setMethod( # make_crsr_graph ----
  f = "make_crsr_graph",
  signature = "Frag.object",
  definition = function(object)
  {
    if(object@frag_type != "crsr"){
      stop("Connection and similarity relationships are required for this function ('cr' and 'sr' arguments).")
    }
    
    cr.net <- make_cr_graph(object)
    sr.net <- make_sr_graph(object)
    
    crsr.net <- igraph::union(cr.net, sr.net)
    crsr.list <- igraph::decompose(crsr.net) # get one graph for each component
    # union of each graph (ie: connection graph) with its complement graph (ie: similarity graph)
    crsr.list <- lapply(crsr.list,
                            function(x) igraph::union(x, igraph::complementer(x)) )
    crsr.list <- lapply(crsr.list,
                            function(x){ igraph::set_vertex_attr(x, "name_save", 
                                                         igraph::V(x),  igraph::V(x)$name )} )
    # merge all the graphs in the list:
    crsr.net <- Reduce(igraph::union, crsr.list)
    
    # ATTRIBUTES ----
    # 1. vertices attributes ####
    fragments.df <- object@fragments.df
    names(fragments.df)[1] <- "name"
    attributes <- merge(                 # retrieve the crsr.net graph attributes
      cbind("name" = igraph::V(crsr.net)$name),
      cbind(fragments.df),
      by="name", sort=FALSE)
    igraph::vertex_attr(crsr.net) <- lapply(attributes, as.character) #add vertex attributes
    
    # 2. edge attributes ####
    igraph::edge_attr(crsr.net) <- list()  # removing all edges attributes
    # we compare the edges in cr.net and crsr.net and only keep the "connection" edges
    # we got a list of the crsr.net edges also present in cr.net
    rename.edges <- function(edge.list){
      chain <- paste(edge.list[[1]], edge.list[[2]], sep="-")
      return(chain)
    }
    crsr.net.edgelist <- igraph::as_edgelist(crsr.net)
    crsr.net.edgelist <- apply(crsr.net.edgelist, 1, rename.edges)
    cr.net.edgelist <- igraph::as_edgelist(cr.net)
    cr.net.edgelist <- apply(cr.net.edgelist, 1, rename.edges)
    
    # setting the "type_relation" edge attribute to "cr"
    igraph::E(crsr.net)$type_relation <- NA
    igraph::E(crsr.net)[ which(crsr.net.edgelist %in% cr.net.edgelist) ]$type_relation <- "cr"
    igraph::E(crsr.net)[ is.na( igraph::E(crsr.net)$type_relation ) ]$type_relation <- "sr"
    igraph::graph_attr(crsr.net) <- list()
    crsr.net <- igraph::set_graph_attr(crsr.net, "frag_type", "connection and similarity relations")
    
    return(crsr.net) 
  }
)


# Class constructor
make_frag_object <- function(cr, sr, fragments)
{
  if( missing(fragments) ){
    stop("A matrix or a data frame is required for the 'fragments' parameter.")
  } 
     
  if( ! missing(cr) & ! missing(sr) ){
    cr[, c(1, 2)] <- apply(cr[, c(1, 2)], 2, as.character)
    cr <- as.matrix(cr)
    sr <- as.matrix(sr)
    sr <- apply(sr, 2, as.character)
    frag_type <- "crsr"
  } else if( ! missing(cr) ){
    cr[, c(1, 2)] <- apply(cr[, c(1, 2)], 2, as.character)
    cr <- as.matrix(cr)
    sr <- matrix()
    frag_type <- "cr"
  } else if( ! missing(sr) ){
    sr <- as.matrix(sr)
    sr <- apply(sr, 2, as.character)
    cr <- matrix()
    frag_type <- "sr"
  }   
  
  fragments[, 1] <- as.character(fragments[, 1])
  
  new(Class="Frag.object", df.cr=cr, df.sr=sr, fragments.df=fragments, frag_type=frag_type)
}

test_that("make_frag_object (only connection relationships)", {
  cr.df <- matrix(c(1,2, 1,3, 2,3, 4,5, 4,6, 7,8), ncol=2, byrow=TRUE)
  sr.df <- matrix( c(1,1, 9,1, 10,1, 11,2, 12,2, 13,2), ncol=2, byrow=TRUE)
  fragments.df <- data.frame(1:13, letters[1:13]) 
  
  f.obj <- make_frag_object(cr=cr.df, fragments=fragments.df)
  expect_equal(class(f.obj)[1], "Frag.object")
  expect_equal(typeof(f.obj), "S4")
})

test_that("make_frag_object (connection and similarity relationships)", {
  cr.df <- matrix(c(1,2, 1,3, 2,3, 4,5, 4,6, 7,8), ncol=2, byrow=TRUE)
  sr.df <- matrix( c(1,1, 9,1, 10,1, 11,2, 12,2, 13,2), ncol=2, byrow=TRUE)
  fragments.df <- data.frame(1:13, letters[1:13]) 
  
  f.obj <- make_frag_object(cr=cr.df, sr=sr.df, fragments=fragments.df)
  expect_equal(class(f.obj)[1], "Frag.object")
  expect_equal(typeof(f.obj), "S4")
})

test_that("make_sr_graph works", {
  sr.df <- matrix( c(1,1, 9,1, 10,1, 11,2, 12,2, 13,2), ncol=2, byrow=TRUE)
  fragments.df <- data.frame(1:13, letters[1:13]) 
  
  f.obj <- make_frag_object(sr=sr.df, fragments=fragments.df)
  g <- make_sr_graph(f.obj)
  
  expect_equal(class(g), "igraph")
  expect_equal(igraph::gorder(g), 6)
  expect_equal(igraph::gsize(g), 6)
})


test_that("make_crsr_graph works", {
  cr.df <- matrix(c(1,2, 1,3, 2,3, 4,5, 4,6, 7,8), ncol=2, byrow=TRUE)
  sr.df <- matrix( c(1,1, 9,1, 10,1, 11,2, 12,2, 13,2), ncol=2, byrow=TRUE)
  fragments.df <- data.frame(1:13, letters[1:13]) 
  
  f.obj <- make_frag_object(cr=cr.df, sr=sr.df, fragments=fragments.df)
  g <- make_crsr_graph(f.obj)
  
  expect_equal(class(g), "igraph")
  expect_equal(igraph::gorder(g), 13)
  expect_equal(igraph::gsize(g), 17)
})


test_that("make_cr_graph works", {
  fragments.df <- data.frame(matrix(c(1,1, 2,1, 3,1, 4,1, 5,1, 6,1, 7,2, 8,2,
                                      9,1, 10,1, 11,2, 12,2, 13,1, 14,3, 15,1,
                                      16,1, 17,2, 18,1, 19,1, 20,1, 21,1, 22,1,
                                      23,3, 24,3, 25,2, 26,2, 27,2, 28,2, 29,2,
                                      30,2, 31,2, 32,2), ncol=2, byrow=T))
  colnames(fragments.df) <- c("node", "layer")
  edges <- data.frame(matrix(c(1,2, 3,4, 4,5, 6,7, 7,8, 8,6, 9,10,
                               10,11, 11,12, 12,9, 13,14, 14,15, 13,15,
                               15,16, 14,17, 18,19, 19,20, 20,18, 19,21,
                               22,23, 23,24, 24,25, 26,27, 27,28, 28,29,
                               26,29, 29,32, 31,32, 31,30, 30,29, 31,29),
                             ncol=2, byrow=T))
  g <- make_frag_object(cr=edges, fragments=fragments.df)
  g <- make_cr_graph(g)
  
  expect_equal(class(g), "igraph")
  expect_equal(igraph::gorder(g), 32)
  expect_equal(igraph::gsize(g), 31)
})  

test_that("retrieved parameters are correct", {
  fragments.df <- data.frame(matrix(c(1,1, 2,1, 3,1, 4,1, 5,1, 6,1, 7,2, 8,2,
                                      9,1, 10,1, 11,2, 12,2, 13,1, 14,1, 15,1,
                                      16,1, 17,2, 18,1, 19,1, 20,1, 21,1, 22,1,
                                      23,1, 24,2, 25,2, 26,2, 27,2, 28,2, 29,2,
                                      30,2, 31,2, 32,2), ncol=2, byrow=T))
  colnames(fragments.df) <- c("node", "layer")
  edges <- data.frame(matrix(c(1,2, 3,4, 4,5, 6,7, 7,8, 8,6, 9,10,
                               10,11, 11,12, 12,9, 13,14, 14,15, 13,15,
                               15,16, 14,17, 18,19, 19,20, 20,18, 19,21,
                               22,23, 23,24, 24,25, 26,27, 27,28, 28,29,
                               26,29, 29,32, 31,32, 31,30, 30,29, 31,29),
                             ncol=2, byrow=T))
  g <- make_frag_object(cr=edges, fragments=fragments.df)
  g <- make_cr_graph(g)
  
  expect_equal(frag.get.parameters(g, "layer", verbose = FALSE)$n.components, 8)
  expect_equal(frag.get.parameters(g, "layer", verbose = FALSE)$vertices, 32)
  expect_equal(frag.get.parameters(g, "layer", verbose = FALSE)$edges, 31)
  expect_equal(frag.get.parameters(g, "layer", verbose = FALSE)$balance, 0.57)
  expect_equal(frag.get.parameters(g, "layer", verbose = FALSE)$components.balance, 0.6)
  expect_equal(frag.get.parameters(g, "layer", verbose = FALSE)$disturbance, 0.19)
  expect_equal(frag.get.parameters(g, "layer", verbose = FALSE)$aggreg.factor, 0.71)
  
  if ( requireNamespace("RBGL", quietly=TRUE)  ) { #  due to issues with RBGL on Win-build 
    expect_equal(frag.get.parameters(g, "layer", verbose = FALSE)$planar, TRUE) 
  } else {
    expect_equal(frag.get.parameters(g, "layer", verbose = FALSE)$planar, NA) 
  }
})



test_that("balance, disturbance, componetn balance is not computed for graph with 1 spatial unit", {
  fragments.df <- data.frame(matrix(c(1,1, 2,1, 3,1, 4,1, 5,1, 6,1, 7,2, 8,2,
                                      9,1, 10,1, 11,2, 12,2, 13,1, 14,1, 15,1,
                                      16,1, 17,2, 18,1, 19,1, 20,1, 21,1, 22,1,
                                      23,1, 24,2, 25,2, 26,2, 27,2, 28,2, 29,2,
                                      30,2, 31,2, 32,2), ncol=2, byrow=T))
  colnames(fragments.df) <- c("node", "layer")
  edges <- data.frame(matrix(c(1,2, 3,4, 4,5, 6,7, 7,8, 8,6, 9,10,
                               10,11, 11,12, 12,9, 13,14, 14,15, 13,15,
                               15,16, 14,17, 18,19, 19,20, 20,18, 19,21,
                               22,23, 23,24, 24,25, 26,27, 27,28, 28,29,
                               26,29, 29,32, 31,32, 31,30, 30,29, 31,29),
                             ncol=2, byrow=T))
  g <- make_frag_object(cr=edges, fragments=fragments.df)
  g <- make_cr_graph(g)
  
  igraph::V(g)$layer <- 1
  
  expect_equal(is.na(frag.get.parameters(g, "layer", verbose = FALSE)$balance), TRUE)
  expect_equal(is.na(frag.get.parameters(g, "layer", verbose = FALSE)$components.balance), TRUE)
  expect_equal(is.na(frag.get.parameters(g, "layer", verbose = FALSE)$disturbance), TRUE)
})

frag.relations.by.layers <- function (graph, layer.attr) 
{
  .check.frag.graph(graph)
  .check.layer.argument(graph, layer.attr)
  layers <- igraph::vertex_attr(graph, layer.attr)
  if (is.null(igraph::V(graph)$name)) {
    igraph::V(graph)$name <- 1:igraph::gorder(graph)
  }
  v.list <- data.frame(v = igraph::V(graph)$name, layer = layers)
  e.list <- data.frame(igraph::as_edgelist(graph))
  e.list <- merge(e.list, v.list, by.x = "X2", by.y = "v")
  e.list <- merge(e.list, v.list, by.x = "X1", by.y = "v")
  res <- table(e.list$layer.x, e.list$layer.y)
  
  row.to.add <- colnames(res)[ ! colnames(res) %in% rownames(res)]
  col.to.add <- rownames(res)[ ! rownames(res) %in% colnames(res)]
  
  # add missing rows and columns
  for(row.name in row.to.add){
    res <- rbind(res, 0)
    rownames(res)[nrow(res)] <- row.name
  }
  for(col.name in col.to.add){
    res <- cbind(res, 0)
    colnames(res)[ncol(res)] <- col.name
  }
  # reorder rows and colums:
  res <- res[order(rownames(res)), order(colnames(res))]  
  
  # sum the upper and lower values and keep the lower triangle of the matrix:
  res[lower.tri(res)] <- res[upper.tri(res)] + res[lower.tri(res)]
  res[upper.tri(res)] <- NA
  
  res
}

frag.get.layers.pair  <- function(graph, layer.attr, sel.layers, size.mini=2, mixed.components.only=FALSE, verbose=TRUE)
{
  # tests:
  .check.frag.graph(graph)
  .check.layer.argument(graph, layer.attr)
  
  if(verbose & ! is.logical(mixed.components.only)) stop("The 'mixed.components.only' parameter requires a logical value.")
  if(verbose & ! is.numeric(size.mini)) stop("The 'size.mini' parameter requires a numerical value.")
  
  igraph::V(graph)$tmp <- igraph::vertex_attr(graph, layer.attr)
  
  if(verbose & (sum(sel.layers %in% igraph::V(graph)$tmp) != 2)){
    stop(paste(c("The values '", sel.layers[1], "' and/or '", sel.layers[2], "' are missing in the '",  layer.attr, "' vertices attribute."),
               sep=" ", collapse = ""))
  }
  # main function:
  subgraph <- igraph::induced_subgraph(graph,
                                       igraph::V(graph)[ igraph::V(graph)$tmp %in% sel.layers ])
  
  igraph::V(subgraph)$membership <- igraph::components(subgraph)$membership
  
  g.list <- igraph::decompose(subgraph)   
  
  if(mixed.components.only == TRUE ){
    sel.components <- sapply(g.list, function(x){
      (length(unique(V(x)$tmp)) != 1)  & (igraph::gorder(x) >= size.mini)
    })
  }else{
    sel.components <- sapply(g.list, function(x) igraph::gorder(x) >= size.mini)
  }
  
  g <- igraph::induced_subgraph(subgraph, 
             igraph::V(subgraph)[igraph::V(subgraph)$membership %in% which(sel.components)])
  if(igraph::gorder(g) == 0){
    if(verbose) {warning("There is no mixed component between these layers.")}
    return(NULL)
  }
  # remove vertex attribute and return result:
  igraph::delete_vertex_attr(g, "tmp")
}

.euclidean.distance <- function(graph, x, y, z){
  coords <- cbind("name" = igraph::V(graph)$name,
                  "x" = igraph::vertex_attr(graph, x),
                  "y" = igraph::vertex_attr(graph, y),
                  "z" = igraph::vertex_attr(graph, z))
  e.list <- igraph::as_edgelist(graph)
  colnames(e.list) <- c("name1", "name2")
  e.list <- merge(e.list, coords, by.x="name1", by.y="name", sort=FALSE)
  e.list <- merge(e.list, coords, by.x="name2", by.y="name",
                  suffixes=c("1", "2"), sort=FALSE)
  e.list <- e.list[, order(colnames(e.list))]
  e.list <- e.list[, -c(1, 2)] 
  e.list <- apply(e.list, 2, as.numeric)
  e.list <- matrix(e.list, ncol = (ncol(coords)-1)*2, byrow=T)
  
  if(any(is.na(e.list))){
    stop("Cannot compute distances, missing coordinates values.")
  }
  
  if( ncol(e.list) == 4){
    sqrt((e.list[,1] - e.list[,2])^2 + (e.list[,3] - e.list[,4])^2 )
  } else if(ncol(e.list) == 6){
    sqrt((e.list[,1] - e.list[,2])^2 + (e.list[,3] - e.list[,4])^2 + (e.list[,5] - e.list[,6])^2 )
  }
}

.get.morpho.spatial.params <- function(graph, x, y, z){
  # input: a graph and the names for x y z attributes
  # output: a data frame with the distance and morpho parameters for the graph edges
  e.list <- igraph::as_edgelist(graph)
  e.list <- data.frame(e.list) 
  e.list$id <- seq_len(nrow(e.list)) # add it to sort the row later
  colnames(e.list) <- c("name1", "name2", "id")
  morpho.df <- cbind("name" = igraph::V(graph)$name, 
                     "morphometry" = igraph::V(graph)$morphometry)
  e.list <- merge(e.list, morpho.df, by.x="name1", by.y="name", sort=FALSE)
  e.list <- merge(e.list, morpho.df, by.x="name2", by.y="name",
                  suffixes=c("1", "2"), sort=FALSE)
  e.list <- e.list[order(e.list$id), ] # sort the rows
  e.list <- e.list[, c(4, 5)]
  e.list <- apply(e.list, 2, as.numeric)
  e.list <- data.frame(matrix(e.list, ncol=2))
  colnames(e.list) <- c("morphometry1", "morphometry2")
  
  e.list$proportion <- apply(e.list, 1, function(xx){
    v <- sort(xx[c(1, 2)])
    v[1] / v[2]
  })
  
  # compute spatial distances (this is done in this function to avoid repeating 3 times the command)
  e.list$distance <- 1 # default value
  coords <- c(x, y, z)[ c(x, y, z) %in% igraph::vertex_attr_names(graph) ]
  if( length(coords) > 1) {   # if at least two coordinates
    e.list$distance <- .euclidean.distance(graph, x, y, z) + 1 # +1 to ensure a neutral value when all distances=0
  }
  e.list
}

# compute morpho-spatial factor:
.morpho.spatial.factor <- function(morpho1, morpho2, proportion, distance,
                                   morpho.max, proportion.max,
                                   distance.max){
  morpho.factor <- (morpho1 + morpho2) / (morpho.max) # +1)
  morpho.prop.factor <- proportion / proportion.max
  distance.factor <- distance / (distance.max) #  +1)
  1 - sqrt(morpho.factor  * sqrt(morpho.prop.factor) * distance.factor)
}

.set.edge.weight <- function(g){
  #edges weight = sum of the degrees * transitivity-based index
  if(igraph::gsize(g) == 0 )  return(0)
  e.list <- igraph::as_edgelist(g)
  colnames(e.list) <- c("src", "tgt")
  v.list <- cbind("name" = igraph::V(g)$name,
                  "deg" = igraph::degree(g),
                  "trans" = igraph::transitivity(g, type = "localundirected",
                                       isolates = "zero", weights = NULL))
  e.list <- merge(e.list, v.list, by.x="src", by.y="name", sort=FALSE)
  e.list <- merge(e.list, v.list, by.x="tgt", by.y="name", sort=FALSE)
  e.list[, 3:6] <- apply(e.list[, 3:6], 2, as.numeric)
  
  e.list$SumDegree <- e.list$deg.x + e.list$deg.y
  e.list$trans.factor <- apply(e.list[, c(4,6)], 1, function(trans){
    3 - (2 / (1 + mean(trans)) )  
  })
  size.factor <- (1 - (1 / ( sqrt(igraph::gorder(g) + igraph::gsize(g)) ) ))^2
  e.list$SumDegree * e.list$trans.factor * size.factor
}




frag.edges.weighting <- function(graph, layer.attr, morphometry=NULL, x=NULL, y=NULL, z=NULL, verbose=TRUE){
  # TESTS: ----
  .check.frag.graph(graph)
  .check.layer.argument(graph, layer.attr)
  
  # test morphometry ----
  igraph::V(graph)$morphometry <- 1 # set default value
  
  if(is.null(morphometry)){
    morphometry <- NULL
    } else if(verbose & ! is.character(morphometry)){
      stop("A character value is expected for the 'morphometry' parameter.")
    } else if( any(igraph::vertex_attr_names(graph) == morphometry) ){
      igraph::V(graph)$morphometry <- igraph::vertex_attr(graph, morphometry)
      morphometry <- "morphometry"
    } else if(verbose) {
      warning(paste("No '", morphometry, "' variable to use for morphometry, parameter skiped.", sep=""))
      morphometry <- NULL
    } else{
      morphometry <- NULL
    }
  
  if(verbose & ! is.numeric(igraph::V(graph)$morphometry)){
    warning("Numeric values are required for the 'morphometry' parameter, parameter skiped.")
    igraph::V(graph)$morphometry <- 1
    morphometry <- NULL
  }
  
  # test coordinates ----
  missing.coords <- c(x, y, z)[ ! c(x, y, z) %in% c("", igraph::vertex_attr_names(graph)) ]
  if(verbose & length(missing.coords) > 0){
    warning(paste0("No '", paste0(missing.coords, collapse = "', '"), "' variable(s) to use for coordinates, parameter skiped."))
  }
  
  if(is.null(x)){
    x <- ""
  } else if(verbose & ! is.character(x)){
    stop("A character value is expected for the 'x' parameter.")
  } else if( ! any(igraph::vertex_attr_names(graph) == x) ){
    x <- ""
  }
  
  if(is.null(y)){
    y <- ""
  } else if(verbose & ! is.character(y)){
    stop("A character value is expected for the 'y' parameter.")
  }  else if( ! any(igraph::vertex_attr_names(graph) == y) ){
    y <- ""
  }
  
  if(is.null(z)){
    z <- ""
  } else if(verbose & ! is.character(z)){
    stop("A character value is expected for the 'z' parameter.")
  }  else if( ! any(igraph::vertex_attr_names(graph) == z) ){
    z <- ""
  }
    
  
  # test spatial units ----
  layers <- igraph::vertex_attr(graph, layer.attr)
  layers.u <- unique(layers)
  if(verbose & length(layers.u) > 2){
    stop("There are more than two layers.")
  }
  
  
  # add unique identifiers:
  if(is.null(V(graph)$name)){
    igraph::V(graph)$name <- seq_len(igraph::gorder(graph))
  }
  
  igraph::E(graph)$id <- seq_len(igraph::gsize(graph))
  
  # generate subgraphs ----
  v1 <- layers == layers.u[1]
  v2 <- layers == layers.u[2]
  
  g1  <- igraph::subgraph_from_edges(graph, igraph::E(graph)[ igraph::V(graph)[v1] %--% igraph::V(graph)[v1] ])
  g2  <- igraph::subgraph_from_edges(graph, igraph::E(graph)[ igraph::V(graph)[v2] %--% igraph::V(graph)[v2] ])
  g12 <- igraph::subgraph_from_edges(graph, igraph::E(graph)[ igraph::V(graph)[v1] %--% igraph::V(graph)[v2] ])
  
  # extract edges indices:
  e1  <- igraph::E(graph)$id %in% igraph::E(g1)$id
  e2  <- igraph::E(graph)$id %in% igraph::E(g2)$id
  e12 <- igraph::E(graph)$id %in% igraph::E(g12)$id
  
  # compute weights for the edges of the 3 subgraphs: ----
  igraph::E(graph)[ e1 ]$weight <- .set.edge.weight(g1)
  igraph::E(graph)[ e2 ]$weight <- .set.edge.weight(g2)
  igraph::E(graph)[ e12]$weight <- .set.edge.weight(g12)
  
  # tags edges ----
  igraph::E(graph)$scope <- "intra"
  igraph::E(graph)[e12]$scope <- "extra"
  
  # RETURN ----
  # IF the morpho-spatial modifier is not required, return the result:
  if(is.null(morphometry) & all(c(x, y, z) == "")){ return(graph) }
  
  # ELSE, compute and apply the morpho-spatial modifier:
  # Morpho-spatial modifier ----
  
  # get the max morphometric/spatial values observed in the whole graph:
  params  <- .get.morpho.spatial.params(graph, x, y, z)
  if(all(params == 1)) {return(graph)}
  
  morph.max <- max(params$morphometry1 + params$morphometry2) 
  prop.max <- max(params$proportion)
  dist.max <- max(params$distance)
  
  # add a "distance" edge attribute if necessary:
  if(! is.null(params$distance)){
    igraph::E(graph)$distance <- params$distance
  }
  
  # get the morphometric/spatial parameters of the edges compute the 
  # morphometric/spatial factor for the edges  of each subgraphs:
  morpho.spatial.factor.g1 <- 0
  if(igraph::gsize(g1) > 0){
    params1  <- .get.morpho.spatial.params(g1, x, y, z)
    morpho.spatial.factor.g1 <- apply(params1, 1, function(xx){
      .morpho.spatial.factor(morpho1 = xx[1],
                             morpho2 = xx[2],
                             proportion = xx[3],
                             distance = xx[4],
                             morpho.max = morph.max,
                             proportion.max = prop.max,
                             distance.max = dist.max)
    })
  }
  
  morpho.spatial.factor.g2 <- 0
  if(igraph::gsize(g2) > 0){
    params2  <- .get.morpho.spatial.params(g2, x, y, z)
    morpho.spatial.factor.g2 <- apply(params2, 1, function(xx){
      .morpho.spatial.factor(morpho1 = xx[1],
                             morpho2 = xx[2],
                             proportion = xx[3],
                             distance = xx[4],
                             morpho.max = morph.max,
                             proportion.max = prop.max,
                             distance.max = dist.max)
    })
  }
  
  morpho.spatial.factor.g12 <- 0
  if(igraph::gsize(g12) > 0){
    params12 <- .get.morpho.spatial.params(g12, x, y, z)
    morpho.spatial.factor.g12 <- apply(params12, 1, function(xx){
      .morpho.spatial.factor(morpho1 = xx[1],
                             morpho2 = xx[2],
                             proportion = xx[3],
                             distance = xx[4],
                             morpho.max = morph.max,
                             proportion.max = prop.max,
                             distance.max = dist.max)
    })
  }
  
  igraph::E(graph)[ e1 ]$weight <- igraph::E(graph)[e1]$weight  * morpho.spatial.factor.g1
  igraph::E(graph)[ e2 ]$weight <- igraph::E(graph)[e2]$weight  * morpho.spatial.factor.g2
  igraph::E(graph)[ e12]$weight <- igraph::E(graph)[e12]$weight * morpho.spatial.factor.g12
  
  graph
}

frag.get.parameters <- function(graph, layer.attr, verbose = TRUE){
  # tests:
  .check.frag.graph(graph)
  .check.layer.argument(graph, layer.attr)
  
  # retrieve and format 'the layer' attribute:
  igraph::V(graph)$layer <- igraph::vertex_attr(graph, layer.attr)
  
  # add edge weight attribute is absent (to avoid an issue with the as_graphnel function):
  if(is.null(igraph::edge_attr(graph, "weight"))){
    igraph::E(graph)$weight <- 1
  }
  
  # test if there are two layers. If true, compute balance, components.balance, disturbance. Else return NAs
  if(length(unique(igraph::V(graph)$layer)) == 2) {
      # fragments balance ----
      balance <- .fragments.balance(graph)
      
      # components balance ----
      compo.balance <- .components.balance(graph)
      
      # estimated disturbance ----
      # proportion of fragments which might have move, determined from the components with fragments from 2 spatial units unequally represented:
      g.list <- frag.get.layers.pair(graph, "layer", unique(igraph::V(graph)$layer), mixed.components.only = TRUE, verbose = verbose)
      disturbance <- 0
      if(! is.null(g.list)){
        g.list <- igraph::decompose(g.list)
        g.list <- sapply(g.list, function(x) 
          table(factor(igraph::V(x)$layer, levels = unique(igraph::V(graph)$layer))) )
        # replace the count of the more represented layer in each component by NA:
        g.list <- apply(g.list, 2, function(x){ x[order(x)][2] <- NA ; x })
        # sum of the count of vertices for the less represented layer in each component:
        disturbance <- sum(g.list, na.rm = TRUE) / igraph::gorder(graph)
        disturbance <- round(disturbance, 2)
      } # end is.null(g.list)
  } else {
    balance <- NA
    disturbance <- NA
    compo.balance <- NA
    if(verbose) warning("The graph does not have two spatial units: disturbance, component balance, and fragment balance values are not computed.")
  }
  
  # aggregation factor ----
  # degree of aggregation of the edges on the components:
  aggreg.factor <- 1 - 1/(1 + stats::sd(sapply(igraph::decompose(graph), igraph::gsize)))
  aggreg.factor <- round(aggreg.factor, 2)
  
  # planarity  ---- 
  # only if the RBGL package is installed
  is.planar <- NA
  if ( requireNamespace("RBGL", quietly=TRUE)  ) {
    is.planar <- RBGL::boyerMyrvoldPlanarityTest(igraph::as_graphnel(graph))
  } else if(verbose) {
    message("The RBGL package is not installed, the `planarity` value cannot be determinated and returned as NA")
  }
  # list results:
  res <- list("n.components" = igraph::components(graph)$no,
              "vertices" =  igraph::gorder(graph),
              "edges" = igraph::gsize(graph),
              "balance" = balance,
              "components.balance" = compo.balance,
              "disturbance" = disturbance, 
              "aggreg.factor" = aggreg.factor,
              "planar" = is.planar,
              "edge.weights.sum" = sum(igraph::E(graph)$weight),
              "edge.weights.median" = stats::median(igraph::E(graph)$weight),
              "edge.weights.median.abs.dev." = stats::mad(igraph::E(graph)$weight)
              )
  # format and return results:
  lapply(res, c, use.names = FALSE)
}

.detect.cycle <- function(graph, x){
  pattern <- igraph::make_ring(x) # create the pattern to detect:
  cycles.list <- igraph::subgraph_isomorphisms(pattern, graph)
  res <- data.frame()
  if(length(cycles.list) != 0){ 
    res <- do.call(rbind, cycles.list) # convert the list to a data frame
    res <- res[ ! duplicated(t(apply(res, 1, sort))), ]
  }
  res
}

.max.cycles.only <- function(x, res){
  if(x-1 == 0) return(res) # control
  remove.id <- sapply(split(res[[x]], row(res[[x]])), 
                      function(i) sum(i %in% res[[x-1]]) == length(i) )
  res[[x]] <- res[[x]][ ! remove.id, ]
  .max.cycles.only(x-1, res) # recursive call
}

# 
frag.cycles <- function(graph, kmax, max.cycles.only=FALSE) {
  # tests:
  .check.frag.graph(graph)  
  
  if(igraph::graph_attr(graph, "frag_type") == "connection and similarity" ){
    warning("Cycle detection in a 'connection and similarity' fragmentation graph is meaningless.")
  }
  if( ! is.numeric(kmax)){
    stop("The 'k' parameter must be numerical.")
  } else  if(kmax < 3) {
    stop("k must be >= 3")
  }
  if( ! is.logical(max.cycles.only)) stop("The 'max.cycles.only' parameter must be logical.")
  
  results <- lapply(kmax:3, function(x) .detect.cycle(graph, x))
  if(max.cycles.only){
    results <- .max.cycles.only(length(results), results)
  }
  results <- rev(results)
  results <- sapply(1:length(results), function(x) length(c(results[[x]])) / (x + 2) ) 
  names(results) <- sapply(3:kmax, function(x) paste(x, "-cycles", sep=""))
  results
}

 

frag.graph.plot  <- function(graph, layer.attr,  ...){
   # tests:
   .check.frag.graph(graph)
   .check.layer.argument(graph, layer.attr)
    if(is.null(igraph::graph_attr(graph, "frag_type"))) stop("The 'frag_type' graph attribute is missing")
   
   # main function:
    igraph::V(graph)$layers <- igraph::vertex_attr(graph, layer.attr)
    
    nLayers <- length(unique(igraph::V(graph)$layers))
    colors <- c("#BBDF27FF",  "darkorchid4", "darkgoldenrod2","chartreuse3", "darkorange3", "brown3",
                "darksalmon", "firebrick2")
    # default edge color:
    igraph::E(graph)$color <- "grey"
    
    layers <- sort(unique(igraph::V(graph)$layers))
    
    if(igraph::graph_attr(graph, "frag_type") == "connection and similarity relations"){
        graph <- igraph::add_layout_(graph, igraph::with_fr(weights = NULL), igraph::component_wise())
        igraph::E(graph)$color <- as.character(factor(igraph::E(graph)$type_relation, labels = c("green", "gray")))
    } else if(igraph::graph_attr(graph, "frag_type") == "similarity relations"){
      graph <- igraph::add_layout_(graph, igraph::with_fr(weights = NULL))
      igraph::E(graph)$color <- "green"
    } else if(length(layers) == 2){ 
      # prepare coordinates if the graph has two layers:
      coords <- data.frame(layer = igraph::V(graph)$layers, miny = 0, maxy = 100) 
      coords[coords$layer == layers[1],]$miny <- 51 
      coords[coords$layer == layers[2],]$maxy <- 49 
      graph$layout <- igraph::layout_with_fr(graph,  niter = 1000, weights =  NULL,
                                     miny = coords$miny, maxy = coords$maxy  )
    }
    
    igraph::plot.igraph(graph, 
         vertex.color = as.character(factor(igraph::V(graph)$layers,
                                            labels = colors[seq_len(nLayers)] )),  
         vertex.label = NA, 
         vertex.size = 4.5,
         edge.width = 2,
         edge.color = igraph::E(graph)$color,
         ...)
}
 

frag.layers.admixture <- function(graph, layer.attr, morphometry=NULL, x=NULL, y=NULL, z=NULL, verbose=TRUE){
  # output : value [0;1]. 0 = "unmixed layers", 1 = "highly mixed layers"
  # tests:
  .check.frag.graph(graph)
  .check.layer.argument(graph, layer.attr)
  
  # extract the user-defined layer attribute and reintegrate it as a vertices attribute named "layer":
  layers <- igraph::vertex_attr(graph, layer.attr)
  igraph::V(graph)$layer <- layers
  layers <- sort(unique(layers))
  
  # Test if empty graph:
  if(length(graph) == 0){
    if(verbose) warning("Empty graph.")
    return(NA)
  }
  
  # Test the number of layers:
  if(length(layers) < 2){
    if(verbose) warning("At least two different layers are required.")
    return(NA)
  }
  
  if(length(layers) == 2){
    if(verbose & is.null(igraph::E(graph)$weight)) stop("The edges must be weighted (using the 'frag.edges.weighting' function).")
    results <- c(admixture = 1 - sum(frag.layers.cohesion(graph, "layer")))
    return(results)
  } else { # if length(layers) > 2
    pairs <- utils::combn(layers, 2) 
    message("More than 2 layers: the 'frag.edges.weighting' function is applied to each pair of layers.")
    results <- sapply(seq_len(ncol(pairs)), function(id){
      gsub <- frag.get.layers.pair(graph, layer.attr, c(pairs[1, id], pairs[2, id]), verbose = T)
      if(is.null(gsub)){
        NA
      } else if(length(unique(V(gsub)$layer)) == 2){
        gsub <- frag.edges.weighting(gsub, layer.attr, morphometry, x, y, z)
        1 - sum(frag.layers.cohesion(gsub, layer.attr))
      } else {
        NA
      }
      
    })
    names(results) <- apply(pairs, 2, function(x) paste(x, collapse = "/"))
    results
  }
}

.cohesion.for.two.layers <- function(g, layers){
  v1 <- igraph::V(g)$layer == layers[1]
  v2 <- igraph::V(g)$layer == layers[2]
  
  # extract three subgraphs:
  g1 <-  igraph::subgraph_from_edges(g, igraph::E(g)[ igraph::V(g)[v1] %--% igraph::V(g)[v1] ])
  g2 <-  igraph::subgraph_from_edges(g, igraph::E(g)[ igraph::V(g)[v2] %--% igraph::V(g)[v2] ])
  g12 <- igraph::subgraph_from_edges(g, igraph::E(g)[ igraph::V(g)[v1] %--% igraph::V(g)[v2] ])
  
  # compute the cohesion values:
  res1 <-  sum(v1, igraph::E(g1)$weight) / sum(igraph::gorder(g), igraph::E(g)$weight)
  res2 <-  sum(v2, igraph::E(g2)$weight) / sum(igraph::gorder(g), igraph::E(g)$weight)
  
  # format and return results:
  c(res1, res2)
}

frag.layers.cohesion <- function(graph, layer.attr, morphometry=NULL, x=NULL, y=NULL, z=NULL, verbose=TRUE){
  # output : value [0;1].
  # tests:
  .check.frag.graph(graph)
  .check.layer.argument(graph, layer.attr)
  
  # delete singletons:
  graph <- igraph::delete_vertices(graph, igraph::degree(graph) == 0)
  
  # Test if empty graph:
  if(length(graph) == 0){
    if(verbose) warning("Empty graph.")
    return(c("cohesion1" = NA, "cohesion2" = NA))
  }
  
  # extract the user-defined layer attribute and reintegrate it as a vertices attribute named "layer":
  layers <- igraph::vertex_attr(graph, name = layer.attr)
  igraph::V(graph)$layer <- layers
  layers <- sort(unique(layers))
  # Test the number of layers :
  if(length(layers) < 2){
    if(verbose) warning("At least two different layers are required.")
    return(c("cohesion1" = NA, "cohesion2" = NA))
  }
  pairs <- utils::combn(layers, 2) 
  
  if(length(layers) == 2){
    if(verbose & is.null(igraph::E(graph)$weight)) stop("The edges must be weighted (using the 'frag.edges.weighting' function).")
    results <- .cohesion.for.two.layers(graph, layers)
    results <- matrix(results)
  } else{ # if length(layers) > 2
    if(verbose) message("More than 2 layers: the 'frag.edges.weighting' function is applied to each pair of layers.")
    
    results <- sapply(seq_len(ncol(pairs)), function(id){
      res <- c("cohesion1" = NA, "cohesion2" = NA)
      gsub <- frag.get.layers.pair(graph, layer.attr, c(pairs[1, id], pairs[2, id]), verbose = verbose)
      if(is.null(gsub)) return(res)
      
      if(length(unique(igraph::V(gsub)$layer)) == 2){
        gsub <- frag.edges.weighting(gsub, layer.attr, morphometry, x, y, z, verbose = verbose)
        res <- .cohesion.for.two.layers(gsub, c(pairs[1, id], pairs[2, id]))
      }
      res
    })
  }
  rownames(results) <- c("cohesion1", "cohesion2")
  colnames(results) <- apply(pairs, 2, function(x) paste(x, collapse = "/"))
  results <- round(results, digits = 4)
  t(results)
}

---
title: "Archeofrag: Refitting and Spatial Analysis in Archaeology"
author: "Sébastien Plutniak"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Archeofrag: Refitting and Spatial Analysis in Archaeology}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
 

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  fig.path = "man/figures/",
  comment = "#>"
)
```

- [**Installation**](#installation)
- [**Building fragmentation graphs**](#building-the-fragmentation-graph)
- [**Edge weighting, cohesion and admixture computation**](#edge-weighting-cohesion-and-admixture-computation)
- [**Testing layer formation hypotheses using simulated data**](#testing-layer-formation-hypotheses-using-simulated data): 
  - [Generating artificial fragmentation graphs](#generating-artificial-fragmentation-graphs)
  - [Testing hypotheses](#testing-hypotheses)
- [**Assessing spatial unit boundaries using similarity relationships**](#assessing-spatial-unit-boundaries-using-similarity-relationships)
- [**Characterising spatial units from their fragmentation**](#characterising-spatial-units-from-their-fragmentation)
 

# Installation
The development version can be installed from GitHub with:

```r
# install.packages("devtools")
devtools::install_github("sebastien-plutniak/archeofrag")
```

Note that *Archeofrag* requires the *RBGL* package available through *Bioconductor*:

```r
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("RBGL")
```

`Archeofrag` is complemented by the `[archeofrag.gui](https://cran.r-project.org/web/packages/archeofrag.gui/index.html)` package, offering a (R Shiny) graphic user interface and additional documentation. For a live  demonstration, click [here](https://analytics.huma-num.fr/Sebastien.Plutniak/archeofrag/).

# Building the fragmentation graph

The `Archeofrag` package comes with a small example data set called “Liang Abu”, related to the pottery fragments found on the surface and in the first two layers of the Liang Abu rock shelter. The data set contains three data frames:

* a table with information about the fragments (a unique identifier, the layer, their length and width, etc.),
* a table with the connection relationships between these fragments (each row contains the unique identifiers of two refitting fragments),
* a table with the similarity relationships between these fragments (two fragments are termed “similar” if they seem to come from the same object but do not have connecting edges. 

The `make_frag_object` function builds objects with the class “frag”. 
Frag objects are not required by the other `Archeofrag` functions, however, using them ensures that the data are suitable for the next steps of the analysis.
The `make_cr_graph` function takes a frag object and generates an `igraph` graph object representing the connection relationships.

```{r make-empirical-graph,  message=F}
library(archeofrag)
abu.frag <- make_frag_object(cr=liangabu.connection, fragments=liangabu.fragments)
abu.g <- make_cr_graph(abu.frag)
```

# Visualisation and subgraph extraction

Several `Archeofrag` functions ensure that the first examination of the data is easy. 
The `frag.relations.by.layers` function returns a matrix with the number of relationships within and between spatial units (e.g., stratigraphic layer).
 
```{r frag.relations.by.layers}
frag.relations.by.layers(abu.g, "layer")
```
The diagonal of the matrix gives the number of intra-layer relationships, and the other values refer to inter-layer relationships. 
Here, for example, there are 31 connection relationships  within layer 2, and 3 connection relationships between layers 1 and 2. No connection relationship was found between the surface (“0”) and layer 2.

The `frag.graph.plot` function generates a visual representation of the graph:

```{r manipulate-plot-abu, fig.align="center"}
par(mar=c(1, 0, 2, 0))
frag.graph.plot(abu.g, layer.attr="layer", main="All layers")
```

The fragments are coloured by layer and the three inter-layer relationships can be observed.

Let us now focus on layers 1 and 2. The `frag.get.layers.pair` function allows the user to extract a pair of layers.

```{r manipulate-get.layer.pair} 
abu.g12 <- frag.get.layers.pair(abu.g, layer.attr="layer",
                                sel.layers=c("1", "2"))
```

This subgraph is drawn with the `frag.graph.plot` function:

```{r manipulate-plot-abu2, fig.align="center"}
par(mar=c(1, 0, 2, 0))
frag.graph.plot(abu.g12, layer.attr="layer", main="Layers 1 and 2")
```

The function has a different behaviour if applied to a fragmentation graph with only two spatial units: the nodes are vertically localised to reflect their location in the two spatial units. In addition, note that standard `plot` arguments can be passed to the `frag.graph.plot` function, e.g., the `main` argument to define the plot's title.

The `frag.get.layers.pair` function has additional parameters to set the minimum size of the connected fragments sets (`size.mini`) and to extract only the sets of connected fragments which include relationships between the two spatial units (`mixed.components.only`). 

```{r manipulate-get.layer.pair2} 
frag.get.layers.pair(abu.g, layer.attr="layer", sel.layers=c("1", "2"),
                     size.mini=2, mixed.components.only=TRUE)
```

Additionally, the `frag.get.layers` function can  extract a set of specified spatial unit(s), e.g., the refits within the first layer at Liang Abu:

```{r manipulate-frag.get.layers}
frag.get.layers(abu.g, layer.attr="layer", sel.layers="1")
```

# Edge weighting, cohesion and admixture computation

Weighting the edges is a crucial step in the TSAR / `Archeofrag` approach because it integrates the topological properties of the fragmentation graph.
The `frag.edges.weighting` function assigns a value to each edge based on the topological properties of the vertices this edge connects.
 
```{r weights-g1}
abu.g12 <- frag.edges.weighting(abu.g12, layer.attr="layer")
```

Then, the `frag.layers.cohesion` function is used to calculate the cohesion value of each layer.

```{r cohesion-g12}
frag.layers.cohesion(abu.g12, layer.attr="layer")
```

These values determine the cohesion (self-adherence) of the spatial units (here, layers) based on the distribution of the refitting relationships.
Note that the weighting of the edges is mandatory for the computation of cohesion. Using the `frag.layers.cohesion` function on a non-weighted fragmentation graph will give an error.

In addition to topological properties, the computation of edge weights can optionally include other parameters, namely the morphometry of the fragments and the distance between the location where they were found.
In the following example, the length of the pottery sherds is used as a morphometric proxy:

```{r weights-g12morpho}
abu.g12morpho <- frag.edges.weighting(abu.g12,
                                      layer.attr="layer",
                                      morphometry="length")
```

Using the morphometry parameter results, layer 2 is more cohesive than layer 1:

```{r cohesion-g12morpho}
frag.layers.cohesion(abu.g12morpho, layer.attr="layer")
```

In addition, the `frag.layers.admixture` function returns a value quantifying the admixture between the two layers. Let us compare the results obtained when the morphometry is used or not:

```{r admixture}
# topology-based weighting:
frag.layers.admixture(abu.g12, layer.attr="layer")
# topology + morphometry weighting:
frag.layers.admixture(abu.g12morpho, layer.attr="layer")
```

In this case, using the morphometry in the computation lowers the admixture between layers 1 and 2 at Liang Abu.

# Testing layer formation hypotheses using simulated data
Simulation-based hypotheses can be tested by combining the functions offered by `Archeofrag`.

## Generating artificial fragmentation graphs

The `frag.simul.process` function generates a pair of spatial units containing fragmented objects with connection relationships within and between these units. 
The next command creates two spatial units populated with 20 initial objects (corresponding to the “connected components” of a graph) which are fragmented into 50 pieces.

```{r make-simulatd-graph, message=FALSE}
simul.g <- frag.simul.process(n.components=20, vertices=50)
```

This illustrates the simplest use of the `frag.simul.process` function, which has several other parameters to control the features of the simulation.

The number of initial spatial units is a crucial parameter, set  using the `initial.layers` parameter with “1” or “2”. This parameter determines the method used to construct the graph and, accordingly, the underlying formation process hypothesis. 

If `initial.layers` is “1”, the fragmentation process is simulated assuming that all the objects were originally buried in a single spatial unit. The two clusters observed at the end of the process are due to fragmentation and displacement.

1. A single spatial unit is populated with the initial objects,
2. the fragmentation process is applied, 
3. spatial units are assigned to the fragments, 
4. some fragments are moved as determined by the value of the `disturbance` parameter.

If `initial.layers` is “2”, it assumes that the objects were buried in two different spatial units, which were later partially mixed due to fragmentation and displacement: 

1. two spatial units are populated with the initial objects (components), 
2. the fragmentation process is applied,
3. disturbance is applied.
 

The `vertices` and `edges` parameters are related: at least one of them must be set, or both (only if `initial.layers` is set to 1). Note that using both parameters at the same time increases the constraints and reduces the number of possible solutions to generate the graph.
When there is no solution, an error occurs and a message suggests how to change the parameters.

The `balance` argument determines the number of fragments in the smaller spatial unit (**before** the application of the disturbance process).
The `components.balance` also determines the contents of the two spatial units by affecting the distribution of the initial objects (components). Note that this argument is used only when `initial.layers` is set to 2.

The `aggreg.factor` parameter affects the distribution of the sizes of the components: this distribution tends to be more unequal when `aggreg.factor` has values close to 1. 

By default, fragments from two spatial units can be disturbed and moved to another other spatial unit. However, the `asymmetric.transport.from` can be used to move fragments from only one given spatial unit.

Finally, the `planar` argument determines if the generated graph has to be planar or not (a graph is planar when it can be drawn on a plane, without edges crossing). This function requires to install the optional `RBGL` package.

An example of a complete configuration of the function is:

```{r simulator-all-params, eval=F}
frag.simul.process(initial.layers=1,
                   n.components=20,
                   vertices=50,
                   edges=40,
                   balance=.4,
                   components.balance=.4,
                   disturbance=.1,
                   aggreg.factor=0,
                   planar=FALSE,
                   asymmetric.transport.from="1")
```

An additional function is intended to simulate the failure of an observer to determine the relationships between fragments. The `frag.observer.failure` function takes a fragmentation graph and randomly removes a given proportion of edges.

```{r frag.observer.failure, eval=FALSE}
frag.observer.failure(abu.g12, likelihood=0.2)
```

## Testing hypotheses 

The versatile `frag.simul.process` function can generate fragmentation graphs under multiple hypotheses about the initial conditions (number of initial objects, number of initial spatial units, etc.).
Testing measurements on observed empirical data against measurements made under these hypotheses can determine the most likely initial conditions and fragmentation process.


Here, this is illustrated by comparing measurements from Liang Abu layers 1 and 2 with measurements from simulated data under two hypotheses about the number of initial spatial units (e.g., layers), using the `initial.layers` parameter with two values, namely one or two initial layers.

A fragmentation graph is generated for each `initial.layers` value, using the parameters observed in the Liang Abu layers 1 and 2 fragmentation graph. 
Setting the  simulator is made easier by using the  `frag.get.parameters` function, which takes a graph and computes a series of parameters that are returned as a list.

```{r params}
params <- frag.get.parameters(abu.g12, layer.attr="layer")
params
```

```{r simulator-test}
# for H2:
test.2layers.g <- frag.simul.process(initial.layers=2,
                                    n.components=params$n.components,
                                    vertices=params$vertices,
                                    disturbance=params$disturbance,
                                    aggreg.factor=params$aggreg.factor,
                                    planar=params$planar)
# for H1:
test.1layer.g <- frag.simul.process(initial.layers=1,
                                   n.components=params$n.components,
                                   vertices=params$vertices,
                                   disturbance=params$disturbance,
                                   aggreg.factor=params$aggreg.factor,
                                   planar=params$planar)
```

Let us now generate not only one graph, but a large number of graphs to statistically compare measurements in the empirical and simulated graphs. 
The `frag.simul.process` function is set for the "two initial layers" hypothesis and embedded into an *ad hoc* function:

```{r simulator-test2, message=FALSE}
run.test2 <- function(x){
  frag.simul.process(initial.layers=2, # note the different value
                     n.components=params$n.components,
                     vertices=params$vertices,
                     disturbance=params$disturbance,
                     aggreg.factor=params$aggreg.factor,
                     planar=params$planar)
}
```

The function is then executed a sufficient number of times:

```{r simulator-test2-run, message=FALSE}
test2.results <- lapply(1:100, run.test2)
```

The empirical values observed for Liang Abu layers 1 and 2 (red line) can now be compared to the values measured in the simulated graph generated under the hypothesis of two initial layers.
This shows, for example, that the empirical admixture value is slightly lower than the simulated admixture values:

```{r manipulate-param-plot-mar, echo=FALSE}
par(mar=c(5, 4, 4, 2))
```

```{r simulator-test2-edges, message=FALSE, fig.align="center", fig.width=4, fig.height=3}
edges.res <- sapply(test2.results,
                    function(g) frag.get.parameters(g, "layer")$edges)
plot(stats::density(edges.res), main="Edges")
abline(v=params$edges, col="red")
```

Similarly, the empirical admixture value is lower than the simulated admixture values:

```{r simulator-test2-admix, message=FALSE, fig.align="center", fig.width=4, fig.height=3}
admix.res <- sapply(test2.results,
                    function(g) frag.layers.admixture(g, "layer"))
plot(stats::density(admix.res), main="Admixture")
abline(v=frag.layers.admixture(abu.g12, "layer"), col="red")
```
 
Two functions (`frag.simul.compare` and `frag.simul.summarise`) facilitate the  execution of the analytical process described above on the initial number of spatial units.
The `frag.simul.compare` function takes an observed fragmentation graph, generates two series of simulated graphs corresponding to two hypotheses on the number of initial spatial units (H1 for one initial spatial unit and H2 for two initial spatial units), and returns a data frame of measurements made on each series (including the edge count, weights sum, balance value, disturbance value, admixture value, and cohesion values of the two spatial units).


```{r simul-compare, message=FALSE}
compare.res <- frag.simul.compare(abu.g12, layer.attr="layer",
                                  iter=30, summarise=FALSE)
head(compare.res$h1.data)
```


For each of these parameters, the `frag.simul.summarise` function facilitates the comparison between empirical observed values and simulated values generated for H1 and H2.


```{r simul-summarise, message=FALSE}
frag.simul.summarise(abu.g12, layer.attr="layer",
                     compare.res$h1.data,
                     compare.res$h2.data)
```

This function returns a data frame with four columns, containing, for each parameter studied:
  
1. whether the series of H1 values are statistically different to the H2 series (Boolean), 
2. the p-value of the Wilcoxon test (numerical), 
3. whether the observed value is "within", "higher", or "lower" to the interquartile range of values for H1, 
4. whether the observed value is "within", "higher", or "lower" to the interquartile range of values for H2.

Note that the `frag.simul.compare` function can optionally be set to execute and return the results of the `frag.simul.summarise` function.




# Assessing spatial unit boundaries using similarity relationships

Similarity relationships are, by construction, not part of the TSAR method, which is based on the topological properties of connection networks. However, since similarity relationships are more frequent in archaeological empirical studies, the `Archeofrag` package includes various functions to handle them. This section illustrates a method to use similarity relationships using `Archeofrag` and R generic functions.

The `make_sr_graph` function takes a “frag” object and  generates an `igraph` similarity network.

```{r make-similarity}
# make a frag object and generate a similarity graph:
abu.frag <- make_frag_object(sr=liangabu.similarity, fragments=liangabu.fragments)
abu.sr <- make_sr_graph(abu.frag)
```

The `frag.relations.by.layers` function returns a table with the number of similarity relationships in and between spatial units, e.g., in the top three layers at Liang Abu:
  
```{r count-similarity}
# count of similarity relationships in and between layers:
simil.by.layers.df <- frag.relations.by.layers(abu.sr, "layer")
simil.by.layers.df
```

These values can be observed as percentages:
  
```{r similarity-perc-tab}
# percentage of similarity relationships in and between layers:
round(simil.by.layers.df / sum(simil.by.layers.df, na.rm=T) * 100, 0)
```

Considering a stratigraphic sequence, adjacent and close layers in the sequence must have lower statistical distances than distant layers. Consequently, it is expected that the result of a hierarchical clustering computed on this distance table would reflect the order of the layers.
The expected result is observed for Liang Abu surface and the first two layers, suggesting an absence of significant disturbance and admixture (\autoref{fig:similarity-dendr-fig}).

```{r similarity-dendr,  eval=T}
# turn similarity into distance:
simil.dist <- max(c(simil.by.layers.df), na.rm=T) - simil.by.layers.df
simil.dist <- as.dist(simil.dist)
# hierarchical clustering:
clust.res <- stats::hclust(simil.dist, method="ward.D2")
```

```{r similarity-dendr-fig, fig.width = 3, fig.height = 3, fig.align="center", fig.cap="Hierarchical clustering of the pottery layers in Liang Abu (distance: based on the number of similarity relationships; clustering method: Ward).", eval=T, message=F}
clust.res$labels <- as.character(factor(clust.res$labels, 
                                        levels=c("0", "1", "2"),
                                        labels=c("layer 0", "layer 1", "layer 2")))
plot(clust.res, hang=-1, axes=F, ann=F)
```


# Characterising spatial units from their fragmentation
The second aim of the TSAR method implemented in `Archeofrag` is to characterise spatial units based on the topological properties of the connection relationships between the fragments they contain. 
Although this aspect is still a work in progress, some functions are already implemented and will be illustrated using simulated data.
The archaeological interpretation of numerical values depends on the type of material (lithic, pottery, etc.) and the completeness or incompleteness of the objects under study and is not discussed here.

```{r subgraphs-simul}
# simulate a fragmentation graph:
simul.g <- frag.simul.process(initial.layers=2,
                              n.components=20,
                              vertices=70,
                              balance=.45)
# extract the subgraph of each spatial unit:
simul.g1 <- frag.get.layers(simul.g, layer.attr="layer", sel.layers="1")[[1]]
simul.g2 <- frag.get.layers(simul.g, layer.attr="layer", sel.layers="2")[[1]]
```

In a graph, a cycle is a path in which only the first and last vertices are repeated.
The `frag.cycles` function searches for cycles in a graph and returns the number of cycles found for different cycle lengths. 
The `kmax` parameter determines the maximal length of the cycles to search for.
Let us compare the cycles found in the two spatial units of the artificial graph:
  
```{r cycles-simul1}
rbind(
  "unit1" = frag.cycles(simul.g1, kmax=5),
  "unit2" = frag.cycles(simul.g2, kmax=5))
```


The `frag.path.lengths` function returns the distribution of the path lengths in the graph (i.e., the number of edges between each pair of vertices). 
This function returns a vector whose first element is the frequency of the paths of length 1, the second element is the frequency of the paths of length 2, etc. 
If the `cumulative` parameter is set to `TRUE`, the function returns the cumulative relative frequency of the path lengths.

```{r pathlengths}
frag.path.lengths(simul.g1)
frag.path.lengths(simul.g2)
frag.path.lengths(simul.g2, cumulative=T)
```

In a graph, the shortest path between two vertices is the path including the least number of edges. The diameter of a graph is its longest shortest path.
The `frag.diameters` function calculates the diameter of each component of the graph and returns the frequency of the values. If the `cumulative` parameter is set to `TRUE`, the function returns the cumulative relative frequency of the diameters.

```{r diameters}
frag.diameters(simul.g1)
frag.diameters(simul.g2)
```

\name{frag.relations.by.layers}
\alias{frag.relations.by.layers}
\title{
Summary of the connection relationships between fragments within and between spatial units.
}
\description{
Return a matrix with the number of relationships within and between spatial units (e.g., layers).
}
\usage{
frag.relations.by.layers(graph, layer.attr)
}
\arguments{
  \item{graph}{An \code{igraph} object.}
  \item{layer.attr}{Character. The name of the vertices attribute giving the spatial unit of each fragment.}
}
\details{
This function is a useful method to summarise the distribution of connection relationships within and between spatial units (e.g., layers).
}
\value{
A square matrix with the number of connection relationships within and between spatial units. 
}
\author{Sebastien Plutniak <sebastien.plutniak at posteo.net>}
\examples{
g <- frag.simul.process(n.components=20, vertices=50, disturbance=.15)
frag.relations.by.layers(g, "layer")
}
\keyword{summary}

\name{frag.edges.weighting}
\alias{frag.edges.weighting}

\title{Weighting the edges of a fragmentation graph}
\description{Weighting of the edges of an archeofrag fragmentation graph.}
\usage{frag.edges.weighting(graph, layer.attr, morphometry, x, y, z, verbose=TRUE)}

\arguments{
  \item{graph}{An undirected \code{igraph} object.} 
  \item{layer.attr}{Character. The name of the vertex attribute with the layer of the fragments.}
  \item{morphometry}{Character. Optional, the name of the vertex attribute with the morphometric value of the fragments.}
  \item{x}{Character. Optional, the name of the vertex attribute with the "x" coordinate of the fragments.}
  \item{y}{Character. Optional, the name of the vertex attribute with the "y" coordinate of the fragments.}
  \item{z}{Character. Optional, the name of the vertex attribute with the "z" coordinate of the fragments.}
  \item{verbose}{Logical. Whether to print or not warning messages.}
}
\details{
In the framework of the TSAR method, this function weights the edges of a fragmentation graph, before computing the cohesion and admixture values.
The weights are computed from the topological properties  of the connection network and can be modified using  the morphometric properties of the fragments and/or the spatial distance between them. In summary, three different parameters can be used:

  1. topology, the basic weighting method;
  2. morphometry, the length, length by width, surface, volume, etc.
  3. spatial distance, expressed using the metric (or other) system, or an ad hoc relative system.

The function must be applied to a fragmentation graph with two spatial units.
Internal connection relationships (within a spatial unit) and external relationships (between the two spatial units) are distinguished, and their respective edge weights are computed in different ways.
Three subgraphs are first generated, one for each spatial unit and only one for the external relations.

The weight of an intra-spatial units edge (E) is equal to the sum of the degrees (d) of the vertices (i and j) it connects: 

  \deqn{W_{intra(E_{ij})} = d_i + d_j}{W_intra(E_ij)= di + dj}

For an inter-spatial unit edge, the same calculation is made but with a modifier to account for the balance of information available for each spatial unit:  
    
  \deqn{W(E_inter{ij}) = (d_i + d_j) \times \left(3 - \frac{2}{ 1 + (tr_i + tr_j)/2}\right) \times \left(1 - \frac{1}{\sqrt{(V_{sub} + E_{sub}) }}\right)^2}{W_inter(E_ij)= (di + dj) * (3 - (2 / (1 + (trans_i+trans_j)/2)) * (1 - 1/(sqrt(V_sub + E_sub)))^2}

with \code{trans_i} and \code{trans_i} the local transitivity of the vertices i and j, and \code{sqrt(V_sub + E_sub)} the square root of the sum of the vertices count and edge count of the sub-graph.

If the \code{morphometry} and/or coordinates (\code{x}, \code{y}, \code{z}) parameters are provided, the previous formula is modified using a factor which is computed as:

   \deqn{f(E_{ij}) =  1 - \left(\sqrt{  \frac{size_i + size_j}{max(sizes)} \times \sqrt{\frac{\frac{size_i}{size_j}}{max(prop)}} \times \frac{distance_{ij}}{max(distances)}}\ \right) }{f = 1 - sqrt( ((size_i+size_j)/max(sizes)) * sqrt((size_i/size_j)/max(prop)) * (distance_ij/max(distances)) ) }


with \code{size_i} the morphometric value of the smaller fragment, \code{max(sizes)} the maximum sum of morphometric values observed for the pairs of connected fragments in the dataset under study; \code{max(prop)} the maximum proportion between the size values of connected fragments observed in the dataset under study; distance ij the spatial distance between fragments i and j; \code{max(distances)} the maximum euclidean distance observed for the pairs of connected fragments in the dataset under study.
Results of the morphometric-spatial factor range in ]0,1].

Error messages are displayed if the vertex "layer" attribute has more than two layers, and a warning is displayed if one or more of the values for the x, y, z parameters do not exist in the input graph.
}
\value{
The graph, with an additional "weight" edge attribute and, if the distance has been computed, a "distance" edge attribute.
}
\author{Sebastien Plutniak <sebastien.plutniak at posteo.net>}
\seealso{
    \code{\link[igraph]{transitivity}}
}
\examples{
g <- frag.simul.process(n.components=20, vertices=50, disturbance=.15)
frag.edges.weighting(g , "layer")
# with morphometric and spatial parameters:
library(igraph)
V(g)$morpho <- sample(1:20, 50, replace=TRUE)
V(g)$x <- sample(1:100, 50, replace=TRUE)
V(g)$y <- sample(1:100, 50, replace=TRUE)
V(g)$z <- sample(1:100, 50, replace=TRUE)
frag.edges.weighting(g, "layer", "morpho", "x", "y", "z")
}
 
\keyword{build data}

\name{Geelbek}
\docType{data}

\alias{Geelbek}
\alias{geelbek.connection}
\alias{geelbek.fragments}

\title{Dataset: Refitting relationships between archaeological fragmented objects from Geelbek dunes}
\description{
   Geelbek dunes is a Middle and Late Stone Age open air site in South Africa. This dataset describes the refitting 'connection' relationships between bone, lithic, pottery, shell, and eggshell fragments found during surveys. A connection relationship refers to a physical connection between two fragments that were part of the same object. 
    
    The dataset is composed of two tables:
    \itemize{
        \item\code{geelbek.connection}: connection relationships between fragments.
        \item\code{geelbek.fragments}: information about the fragments.
    }
}
\usage{data(Geelbek)}
\format{
    \itemize{
        \item\code{geelbek.connection} is a 1081x2 matrix. Each line describes a connection relationship between two fragments. Their unique identifiers are given in columns 'id1' and 'id2', respectively. 
        \item\code{geelbek.fragments} is a 1458x6 data frame:
        \itemize{
            \item id: fragment unique identifier 
            \item sector: locality in the dunes. 
            \item material: type of objects (bone, lithic, pottery, shell, eggshell)
            \item x: x coordinate of the location where the fragment was found (in metre)
            \item y: y coordinate of the location where the fragment was found (in metre)
            \item z: z coordinate of the location where the fragment was found (in metre)
        }
    }
    
}
\references{
    \itemize{
		   \item Nicholas J. Conard, Andrew W. Kandel,  Sébastien Plutniak. 2025. Refitting archaeological objects from the Geelbek Dunes Middle and Later Stone Age site (South Africa). Zenodo, https://doi.org/10.5281/zenodo.15803288
		   \item Andrew W. Kandel, Nicholas J. Conard. 2012. Settlement patterns during the Earlier and Middle Stone Age around Langebaan Lagoon, Western Cape (South Africa). Quaternary International, 270, 15-29, http://doi.org/10.1016/j.quaint.2011.06.038
}
    }
\keyword{datasets}

\examples{
data(Geelbek)
head(geelbek.fragments)
}

\name{make_frag_object}
\alias{make_frag_object}
\title{
    Makes a "frag.object" object.
}
\description{
    Makes a "\code{\link[=Frag.object-class]{frag.object}}" object.    
    
}
\usage{
make_frag_object(cr, sr, fragments)
}
\arguments{
  \item{cr}{
  A matrix or a data frame with two columns giving the vertex id of each pair of connected fragments.
}
  \item{sr}{
    Optional. A matrix or a data frame with two columns: the first gives the fragment id, the second gives the "similarity group" id. Optional if \code{mode} is "cr".
}
  \item{fragments}{
    A matrix or a data frame with information about each fragment. The first column must contain the fragments' id. 
}

}
\details{This function checks the dataset and returns a "frag.object" which can be turned into a fragmentation graph using the \code{\link{make_cr_graph}}, \code{\link{make_sr_graph}}, or \code{\link{make_crsr_graph}} functions.}
\value{
    An object of "\code{\link[=Frag.object-class]{frag.object}}" class.
}
\author{Sebastien Plutniak <sebastien.plutniak at posteo.net>}
\examples{
cr.df <- matrix(c(1,2, 1,3, 2,3, 4,5, 4,6, 7,8), ncol=2, byrow=TRUE)
sr.df <- matrix( c(1,1, 9,1, 10,1, 11,2, 12,2, 13,2), ncol=2, byrow=TRUE)
fragments.df <- data.frame(1:13, letters[1:13]) 

make_frag_object(cr=cr.df, fragments=fragments.df)
make_frag_object(cr=cr.df, sr=sr.df, fragments=fragments.df)
}
\keyword{build data}

\name{frag.simul.process}
\alias{frag.simul.process}
\title{
Simulate the fragmentation of archaeological objects scattered in two spatial units
}
\description{
Simulate the fragmentation of archaeological objects scattered in two spatial units.
}
\usage{
frag.simul.process(initial.layers=2, n.components=NULL, vertices=Inf,
                   edges=Inf, balance=.5, components.balance=.5,
                   disturbance=0, aggreg.factor=0, planar=FALSE,
                   asymmetric.transport.from=NULL, 
                   from.observed.graph=NULL, observed.layer.attr=NULL, 
                   verbose=TRUE)
}

\arguments{
  \item{initial.layers}{Integer (1 or 2). Number of hypothetical spatial units (e.g. layers) to use as initial condition.}
  \item{n.components}{Integer. Innitial number of objects (connected components).}
  \item{vertices}{Integer. Final number of fragments (vertices).}
  \item{edges}{Integer. Final number of connection relationships between fragments (edges).}
  \item{balance}{Numeric ]0;1[. Proportion of fragments to generate in the first spatial unit before applying disturbance.}
  \item{components.balance}{Numeric ]0;1[. Proportion of objects (connected components) in the first spatial unit before applying fragmentation (only used when initial.layers=2).}
  \item{disturbance}{Numeric [0;1]. Final proportion of fragments moved from a spatial unit to the other.}
  \item{aggreg.factor}{Numeric [0;1]. Higher values increase the likelihood that the biggest components are selected when adding fragments or connections (see details).}
  \item{planar}{Logical. If TRUE, generates a planar graph (if FALSE, the graph can be planar or not).}
  \item{asymmetric.transport.from}{Numeric or character value in "0", "1", and "2". Determine the direction of fragment transport when generating disturbance. Null or "0" value is for bidirectional transport,  "1" and "2" are for asymmetric transport from the first and second spatial units, respectively.}
  \item{from.observed.graph}{igraph object. If not NULL, the parameters observed in this fragmentation graph are used instead of the previous parameters. See details.}
  \item{observed.layer.attr}{character. Required if the \code{from.observed.graph} option is used. Name of the spatial unit attribute in the observed graph.}
  \item{verbose}{Logical. Whether to print or not warning messages.}
}
\details{
This function simulates the fragmentation of archeological objects within and between two adjacent stratigraphic layers.
Fragments are represented by vertices and the "connection" relationships ("refittings") between them are represented by edges.
All fragments have at least one relation ("single" fragments are not generated).

Some parameters are optional or depend on other parameters (messages are displayed accordingly). 
Namely, setting \code{initial.layers}=1 enables to constraint the graph with the number of vertices only, the number of edges only, or both. 
With \code{initial.layers}=2, the \code{components.balance} can be used, and the \code{edges} parameter is not supported (only the \code{vertices} parameter can be used).

The \code{components.balance} parameter determines the proportion of components (i.e. objects) in the first spatial unit before the application of the fragmentation process;
the \code{balance} parameter determines the proportion of fragments in the first spatial unit before the application of the disturbance process.
The \code{disturbance} parameter determines the proportion of fragments to move from one spatial unit to another. Consequently, it generates inter-spatial units relationships. Highest admixture values are generated with disturbance=0.5; disturbance values > 0.5 revert the fragment balance between the two spatial units. 
The \code{aggreg.factor} contraints the selection of the object (set of fragments, i.e. connected components) to be fragmented at each iteration of the simulation: with \code{aggreg.factor=0}, the probability is equal for all objects; with \code{aggreg.factor=0.01}  the probability to select the 1st largest component is 0.99 and the smallest component is 1/4; with \code{aggreg.factor=1} the probability to select the 1st largest component is 1/2, the second largest component is 1/3, the third largest is 1/4, etc.
If \code{asymmetric.transport.from} is set to 1 or 2, the disturbance process is only applied to the fragments from spatial unit 1 or spatial unit 2, respectively.

If a graph is given to the \code{from.observed.graph} parameter, the properties of this graph are internally retrieved with the \code{frag.get.parameters} function (including:  the number of components, number of vertices, balance, the components.balance, the disturbance, the aggregation factor, and whether the graph is planar or not; note that the number of edges is not included as a parameter). If some other parameters of the \code{frag.simul.process} function are set, the values retrieved from the observed graph are used in replacement.
The \code{frag.edges.weighting} is internally applied to weight the graph edges.

Setting the \code{planar} argument to TRUE constraints the graph to be planar (if this parameter is FALSE, the graph can be planar or not). 
Note that using the \code{planar} argument requires to install the optional RBGL package and that the simulator is faster with \code{initial.layers}=2 and  \code{planar}=FALSE.
}

\value{
An igraph object with a "frag_type" graph attribute (with the value "cr", for "connection relationship") and three vertices attributes: "name" (vertices identifiers), "layer" (with the values "1" and "2"), and "object.id" (component identifiers).

}
\author{Sebastien Plutniak <sebastien.plutniak at posteo.net>}
\seealso{
    \code{\link[archeofrag]{frag.get.parameters}},
    \code{\link[archeofrag]{frag.edges.weighting}},
    \code{\link[RBGL]{boyerMyrvoldPlanarityTest}}
}

\examples{
frag.simul.process(n.components=20, vertices=50, disturbance=.15)

g <- frag.simul.process(initial.layers=1, 
                            n.components=20,
                            vertices=50,
                            edges=40,
                            balance=.5,
                            components.balance=.5,
                            disturbance=.1,
                            planar=FALSE)
plot(g, vertex.color=factor(igraph::V(g)$layer), 
     vertex.size=4, vertex.label=NA)

}
\keyword{simulation}

\name{Fumane}
\docType{data}

\alias{Fumane}
\alias{fumane.connection}
\alias{fumane.fragments}

\title{Dataset: Refitting relationships between lithic fragments from the Fumane cave}
\description{
   The Fumane Cave is a Paleolithic archaeological site in Italy. This dataset describes the refitting 'connection' relationships between lithic fragments found during excavations. A connection relationship refers to a physical connection between two fragments that were part of the same object. 
    
    The dataset is composed of two tables:
    \itemize{
        \item\code{fumane.connection}: connection relationships between fragments.
        \item\code{fumane.fragments}: information about the fragments.
    }
}
\usage{data(Fumane)}
\format{
    \itemize{
        \item\code{fumane.connection} is a 261x2 matrix. Each line describes a connection relationship between two fragments. Their unique identifiers are given in column 'id1' and in column 'id2', respectively. 
        \item\code{fumane.fragments} is 508x8 data frame:
        \itemize{
            \item id: fragment unique identifier 
            \item us: stratigraphic layer
            \item square:  excavation grid square where the fragment was found
            \item sub_square:  excavation grid subsquare where the fragment was found
            \item thickness: thickness of the fragment (in millimeter)
            \item length: length of the fragment (in millimeter)
            \item width: width of the fragment (in millimeter)
            \item volume: approximated volume of the fragment, product of the thickness by the length by the width (in cubic millimeter)
        }
    }
    
}
\references{
    \itemize{
		\item  Armando Falcucci. 2025. Refitting-The-Context: Accepted paper b (v0.1.3). Zenodo. https://doi.org/10.5281/zenodo.15017627  
        }
    }
\keyword{datasets}

\examples{
data(Fumane)
head(fumane.fragments)
}

\name{frag.simul.summarise}
\alias{frag.simul.summarise}

\title{Summarise the comparison between an observed fragmentation graph and simulated graphs for two deposition hypotheses.}
\description{Compare the parameters measured on simulated graphs generated for two deposition hypotheses with Wilcoxon tests, and the corresponding values measured on an empirical graph, and returns a summary data frame.}
\usage{frag.simul.summarise(graph, layer.attr, res.h1, res.h2, cohesion1.attr="cohesion1",
                            cohesion2.attr="cohesion2", admixture.attr="admixture",
                            verbose=TRUE)}

\arguments{
  \item{graph}{An undirected \code{igraph} object. The graph to compare with simulated graphs.} 
  \item{layer.attr}{Character. The name of the vertices attribute giving the spatial unit of the fragments (layer or other).}
  \item{res.h1}{data frame. A data frame with the parameters observed on the simulated graphs for H1.}
  \item{res.h2}{data frame. A data frame with the parameters observed on the simulated graphs for H2.}
  \item{cohesion1.attr}{character. The name of the column in the data frames res.h1 and res.h2 with the cohesion values of the spatial unit 1.}
  \item{cohesion2.attr}{character. The name of the column in the data frames res.h1 and res.h2 with the cohesion values of the spatial unit 2.}
  \item{admixture.attr}{character. The name of the column in the data frames res.h1 and res.h2 with the admixture values.}
  \item{verbose}{Logical. Whether to print or not warning messages.}
}
\details{
This function compares and summarises the numerical values measured on the series of graphs generated for the two deposition hypotheses (H1: one initial spatial unit, H2: two initial spatial units). It is intended to post-process the results of the \code{frag.simul.compare} function, but it can also be applied to other inputs.

The data frames with the results for H1 and H2 must have exactly the same column names, corresponding to parameters measured on the simulated graphs. When using the result generated with the \code{frag.simul.compare} function, the parameters considered include: the edge count, the sum of the edge weights, the balance, the components balance, the disturbance, the admixture and the cohesion of the two spatial units). When using this function alone, the names of the columns of the data frames can be set with the cohesion1.attr, cohesion2.attr, and admixture.attr parameters (default values:  "cohesion1", "cohesion2", "admixture" respectively).

For each parameter, a two-sample Wilcoxon test is run to compare the series of values generated for H1 and H2. In addition, the value measured on the observed graph is compared with the range of values generated for the two hypotheses. The results of these comparisons are reported as a data frame with four columns: for each parameter studied, the data frame contains 1) whether the series of H1 values are statistically different to the H2 series (Boolean), 2) the p-value of the Wilcoxon test (numerical), 3) whether the observed value is "within", "higher", or "lower" to the interquartile range of values for H1, 4) whether the observed value is "within", "higher", or "lower" to the interquartile range of values for H2.

}
\value{
A data frame summarising the comparison between the simulated results for the two hypotheses, and the values measured on the empirical graph with the simulated results.

}
\author{Sebastien Plutniak <sebastien.plutniak at posteo.net>}

\seealso{
\code{\link[archeofrag]{frag.simul.compare}},
\code{\link[stats]{wilcox.test}},

}
\examples{
g <- frag.simul.process(n.components=20, vertices=50, disturbance=.15)
g <- frag.edges.weighting(g, layer.attr="layer")
\dontrun{res <- frag.simul.compare(g, layer.attr="layer", iter=30, summarise=FALSE)
frag.simul.summarise(g, layer.attr="layer", res.h1=res[[1]], res.h2=res[[2]])}
}
 
\keyword{simulation}

\name{frag.observer.failure}
\alias{frag.observer.failure}

\title{Simulate the failure of an observer to determine the relationships between fragments.}
\description{Simulate the failure of an observer to determine the relationships between fragments.}
\usage{frag.observer.failure(graph, likelihood, remove.vertices=FALSE)}

\arguments{
  \item{graph}{An undirected \code{igraph} object.} 
  \item{likelihood}{Numerical vector of values in [0,1] giving the likelihood of not observing a relationship between two fragments.}
  \item{remove.vertices}{Logical. If TRUE, unconnected vertices are removed.}
}
\details{
In determining connection relationships between archaeological fragments, archaeologists often consider the likelihood that they fail in identifying some of these relationships.
Given an initial fragmentation graph, this function aims to simulate the effects of such different likelihood values.

For each value in the \code{likelihood} parameter, a new graph is generated by randomly removing the given proportion of edges from the input graph.
When generating series of comparable graphs with different likelihoods, the function internally resets the seed for random number generation, to ensure that edges are removed in the same order.

An error message is displayed if at least one of the \code{likelihood} values is < 0 or > 1.
}
\value{
A list of fragmentation graphs (igraph objects).
}
\author{Sebastien Plutniak <sebastien.plutniak at posteo.net>}

\seealso{
\code{\link[base]{set.seed}}
}
\examples{
g <- frag.simul.process(n.components=20, vertices=50, disturbance=.15)
frag.observer.failure(graph=g, likelihood=c(0.05, 0.1), remove.vertices=FALSE)
}
 
\keyword{simulation}

\name{frag.cohesion.ranking}
\alias{frag.cohesion.ranking}

\title{Rank a series of spatial units by comparing their cohesion values}
\description{Rank a series of spatial units based on how many times they feature the highest cohesion value in pairwise comparisons.}
\usage{frag.cohesion.ranking(cohesion.df, add.math.signs=TRUE)}

\arguments{
  \item{cohesion.df}{Matrix or data.frame, with two columns for the cohesion values for each pair of spatial units and spatial units labesl in row names.} 
  \item{add.math.signs}{Boolean. Wheter adding or not (in)equality  signs.}
}
\details{
Cohesion values computed with the  \code{frag.layers.cohesion()} are relative to the pair of spatial units under consideration and do not allow for comparison of more than two spatial units. The \code{frag.cohesion.ranking} function offers a method to analyse larger series of spatial units. 

Given the cohesion values computed for more than two spatial units, it counts how many times each spatial unit features the highest cohesion value in pairwise comparisons. Spatial units are sorted based on this count.

By default, the function's output is formated for human reading, adding equality and inequality signs in the series of spatial units labels to help in interpeting it.  This enhancement can be disabled with \code{add.math.signs=FALSE}.

The matrix or data frame given to the \code{cohesion.df} parameter must be formated as an output of the \code{frag.layers.cohesion()} function. Namely,  each row corresponds to the comparison of two spatial units, the two columns include the corresponding cohesion values, and the row name gives the labels of the spatial units (separated by the sign '/').
}
\value{
If  \code{add.math.signs=TRUE}, a data frame inclduding (in)equality signs and empty values. Else, a named numeric vector.
}
\author{Sebastien Plutniak <sebastien.plutniak at posteo.net>}
\seealso{
    \code{\link[archeofrag]{frag.layers.cohesion}}
}
\examples{
g1 <- frag.simul.process(n.components=5,  vertices=20, disturbance=.1)
g2 <- frag.simul.process(n.components=10, vertices=90, disturbance=.03)

# assign different names for these spatial units:
igraph::V(g2)$layer <- as.character(factor(igraph::V(g2)$layer, labels = c(3, 4))) 
igraph::V(g2)$name <- paste0(igraph::V(g2)$name, "g2")

g <- igraph::disjoint_union(g1, g2) # merge the graphs
g$frag_type <- "cr" # this value was lost while merging

cohesion.res <- frag.layers.cohesion(g, layer.attr="layer", verbose=FALSE) # compute cohesions
frag.cohesion.ranking(cohesion.res, add.math.signs = TRUE) # rank the spatial units
}
 
\keyword{measurement}

\name{LiangAbu}
\docType{data}

\alias{LiangAbu}
\alias{liangabu.connection}
\alias{liangabu.similarity}
\alias{liangabu.fragments}

\title{Dataset: Archeological relationships between pottery fragments in Liang Abu}
\description{
    Liang Abu is an archaeological site in East Kalimantan, Indonesia. This dataset describes the relationships between pottery fragments found during   excavations (2009-2012). Two types of relationships are defined.
    \itemize{
        \item A connection relationship refers to a physical connection between two fragments that were part of the same object. 
        \item A similarity relationship between fragments is defined if there is an acceptable likelihood that those fragments were part of the same object.
    }
    
    The dataset is composed of three tables:
    \itemize{
        \item\code{liangabu.connection}: "connection" relationships between fragments.
        \item\code{liangabu.similarity}: "similarity" relationships between fragments.
        \item\code{liangabu.fragments}: information about the fragments.
    }
}
\usage{data(LiangAbu)}
\format{
    \itemize{
        \item\code{liangabu.connection} is a 56x2 matrix. Each line describes a connection relationship between two fragments. Their unique identifiers are given in column 'id1' and in column 'id', respectively. 
        \item\code{liangabu.similarity} is a 147x2 matrix. Column 'id' gives a fragment unique identifier, column 'su.id' gives a unique identifier for the group of similar fragments it belongs to (similarity unit).
        \item\code{liangabu.fragments} is 177x11 data frame:
        \itemize{
            \item id: fragment unique identifier
            \item layer:  stratigraphic layer
            \item zmin: minimal depth in centimetres where the fragment was found
            \item zmax: maximal depth in centimetres where the fragment was found
            \item square: grid square where the fragment was found
            \item square.x: x coordinate in centimetres of the middle of the grid square
            \item square.y: y coordinate in centimetres of the middle of the grid square
            \item thickness: thickness of the fragments in millimetres
            \item length: length of the fragments in millimetres
            \item thickness.by.length: product of the thickness by the length
            \item sherd.type: type of pottery sherd
        }
    }
    
}
\references{
    \itemize{
		\item Plutniak, Sebastien, "The Strength of Parthood Ties. Modelling Spatial Units and Fragmented Objects with the TSAR Method - Topological Study of Archaeological Refitting", Journal of Archaeological Science, vol. 136, 105501, doi: 10.1016/j.jas.2021.105501.
        \item Plutniak, Sebastien, "Refitting pottery fragments from the Liang Abu rockshelter, Borneo", Zenodo, doi: 10.5281/zenodo.4719578.
        \item Plutniak, Sebastien, Astolfo Araujo, Simon Puaud, Jean-Georges Ferrie, Adhi Agus Oktaviana, Bambang Sugiyanto, Jean-Michel Chazine et Francois-Xavier Ricaut. 2015. "Borneo as a half empty pot: Pottery assemblage from Liang Abu, East Kalimantan, Quaternary International, doi: 10.1016/j.quaint.2015.11.080.
        }}
\keyword{datasets}

\examples{
data(LiangAbu)
head(liangabu.fragments)
}

\name{frag.layers.admixture}
\alias{frag.layers.admixture}

\title{Admixture of two spatial units}

\description{Evaluate how reliable the distinction is between two archaeological spatial units (layers or any other type of spatial units).}
\usage{frag.layers.admixture(graph, layer.attr, morphometry, x, y, z, verbose=TRUE)}

\arguments{
  \item{graph}{An undirected \code{igraph} object.} 
  \item{layer.attr}{Character. The name of the vertex attribute giving the spatial unit of each fragment (e.g. layer).}
  \item{morphometry}{Character. Optional, to pass to the `frag.edges.weighting` function: name of the vertex attribute with the morphometric value of the fragments.}
  \item{x}{Character. Optional, to pass to the `frag.edges.weighting` function: name of the vertex attribute with the "x" coordinate of the fragments.}
  \item{y}{Character. Optional, to pass to the `frag.edges.weighting` function: name of the vertex attribute with the "y" coordinate of the fragments.}
  \item{z}{Character. Optional, to pass to the `frag.edges.weighting` function:  name of the vertex attribute with the "z" coordinate of the fragments.}  
  \item{verbose}{Logical. Whether to print or not warning messages.}
}
\details{
This function returns a value reflecting the robustness of the distinction between two spatial units (layers or any other relevant archaeological spatial unit).
The admixture value is computed as:

    \code{1 - cohesion(layer 1) - cohesion(layer 2)}

The admixture of two spatial units is equal to the cohesion of a virtual third unit, which is defined by the fragments and the connection relationships intersecting the two spatial units. 
Results range in [0;1] with 0 for two completely independent spatial units and values towards 1 as the robustness of the boundary between the two spatial units is lower.
As it appears, this function calls the \code{frag.layers.cohesion} function.

The basic use of this function is intended for a graph with two layers, whose edges have been previously weighted using the \code{frag.edges.weighting} function (an error message is displayed if the vertice attribute "layer" contains less than two spatial units, and if the graph does not have an edge attribute "weight").

However, the function can also be used for a graph with more than two spatial units. In this case, a subgraph is generated for each pair of layers (using the \code{frag.get.layers.pair} function), the \code{frag.edges.weighting} function is applied to weight their edges (a warning message is displayed), and the admixture is computed.

An error message is displayed if the vertex "layer" attribute has less than two values.
}
\value{
A numeric vector with the admixture of each pair of spatial units.
}
\author{Sebastien Plutniak <sebastien.plutniak at posteo.net>}

\seealso{
\code{\link[archeofrag]{frag.edges.weighting}},
\code{\link[archeofrag]{frag.layers.cohesion}}
}
\examples{
g <- frag.simul.process(n.components=20, vertices=50, disturbance = .15)
g <- frag.edges.weighting(g, layer.attr="layer")
frag.layers.admixture(g, "layer")
}
 
\keyword{measurement}

\name{GrandeRivoire1stMeso}
\docType{data}

\alias{GrandeRivoire1stMeso}
\alias{grande.rivoire.1m.connection}
\alias{grande.rivoire.1m.fragments}

\title{Dataset: Refitting relationships between bone and lithic fragments from the Grande Rivoire site, First Mesolithic levels}
\description{
   Grande Rivoire is a Mesolithic rock shelter located in France. This dataset describes the refitting 'connection' relationships between bone and lithic fragments found during excavations of the First Mesolithic levels. Connection relationship refers to physical connection between two fragments that were part of the same object. 
    
    The dataset is composed of two tables:
    \itemize{
        \item\code{grande.rivoire.1m.connection}: connection relationships between fragments.
        \item\code{grande.rivoire.1m.fragments}: information about the fragments.
    }
}
\usage{data(GrandeRivoire1stMeso)}
\format{
    \itemize{
        \item\code{grande.rivoire.1m.connection} is a 712x2 matrix. Each line describes a connection relationship between two fragments. Their unique identifiers are given in column 'id1' and in column 'id2', respectively. 
        \item\code{grande.rivoire.1m.fragments} is a 1165x5 data frame:
        \itemize{
            \item id: fragment unique identifier 
            \item layer: archaeological layer where the fragment was found
            \item layer.su: combination of the archaeological layer and sedimentary unit where the fragment was found
            \item sector: site sector where the fragment was found
            \item type: type of object (bone or lithic)
        }
    }
    
}
\references{
    \itemize{
		\item Derbord L.,  A. Angelin. 2025. 'Mesolithic artefact refitting data from La Grande Rivoire (Sassenage, Isère)'. Zenodo. https://doi.org/10.5281/zenodo.15289796
        }
    }
\keyword{datasets}

\examples{
data(GrandeRivoire1stMeso)
head(grande.rivoire.1m.fragments)
}

\name{frag.get.layers.pair}
\alias{frag.get.layers.pair}
\title{
Extracts the subgraph corresponding to a pair of archaeological spatial units.
}
\description{
Extracts the subraph corresponding to a pair of archaeological spatial units (stratigraphic layers or any other type of spatial unit).
}
\usage{
frag.get.layers.pair(graph, layer.attr, sel.layers, size.mini=2,
                     mixed.components.only=FALSE, verbose=TRUE)
}

\arguments{
  \item{graph}{An \code{igraph} object.}
  \item{layer.attr}{Character. The name of the vertices attribute giving the spatial unit of each fragment.}
  \item{sel.layers}{A numeric vector of length 2 with the name of the spatial unit selected for extraction.}
  \item{size.mini}{A minimal number of vertices for the components to include in the resulting graph.}
  \item{mixed.components.only}{Logical. If TRUE, only the components with fragments from the two selected spatial units are returned. If FALSE, all the components of the two spatial units are extracted.}
  \item{verbose}{Logical. Whether to print or not warning messages.}
}
\details{
The default setting of the \code{mixed.components.only} argument is FALSE, for convenience for other measurements.
}
\value{
An undirected graph object.
}
\author{Sebastien Plutniak <sebastien.plutniak at posteo.net>}

\examples{
g <- frag.simul.process(n.components=20, vertices=50, disturbance=.15)
igraph::V(g)$layers <- c(rep("layer1", 20), rep("layer2", 20), rep("layer3", 10))

frag.get.layers.pair(g, "layers", sel.layers=c("layer2","layer3"),
                     size.mini=2, mixed.components.only=FALSE)
frag.get.layers.pair(g, "layers", sel.layers=c("layer2","layer3"),  
                     size.mini=2, mixed.components.only=TRUE)
}
\keyword{extraction}

\name{BoutDesVergnes}
\docType{data}

\alias{BoutDesVergnes}
\alias{bdv.connection}
\alias{bdv.fragments}

\title{Dataset: Refitting relationships between lithic fragments from the Bout des Vergnes site}
\description{
   Bout des Vergnes is an open-air Palaeolithic site located in France. This dataset describes the refitting 'connection' relationships between lithic fragments found during excavations. Connection relationship refers to physical connection between two fragments that were part of the same object. 
    
    The dataset is composed of two tables:
    \itemize{
        \item\code{bdv.connection}: connection relationships between fragments.
        \item\code{bdv.fragments}: information about the fragments.
    }
}
\usage{data(BoutDesVergnes)}
\format{
    \itemize{
        \item\code{bdv.connection} is a 1767x2 matrix. Each line describes a connection relationship between two fragments. Their unique identifiers are given in column 'id1' and in column 'id2', respectively. 
        \item\code{bdv.fragments} is 2204x5 data frame:
        \itemize{
            \item id:  fragment unique identifier 
            \item layer: chronocultural unit associated to the fragment was found. Values are numbered in expected chrono-stratigraphic order
            \item x: x coordinate of the location where the lithic fragment was found (in metre)
            \item y: y coordinate of the location where the lithic fragment was found (in metre)
            \item z: z coordinate of the location where the lithic fragment was found (in metre)
        }
    }
    
}
\references{
    \itemize{
		\item  Ihuel, E. (dir.), with  Baillet M., Barbeyron A., Brenet M., Camus H., Claud E., Mercier N., Michel A., Sellami F. 2020. Le Bout des Vergnes, Bergerac (Dordogne, Nouvelle-Aquitaine), Contournement ouest de Bergerac, RD 709, Rapport final d’opération, Périgueux, Conseil départemental de la Dordogne, Service départemental d'archéologie.
        }
    }
\keyword{datasets}

\examples{
data(BoutDesVergnes)
head(bdv.fragments)
}

\name{frag.cycles}
\alias{frag.cycles}
\title{
Count the k-cycles in a graph, for cycles =< k
}
\description{
Count the k-cycles in a graph, for cycles =< k
}
\usage{
frag.cycles(graph, kmax, max.cycles.only=FALSE)
}
\arguments{
  \item{graph}{An \code{igraph} object, must be an undirected graph.}
  \item{kmax}{Maximal length of the cycles to detect.}
  \item{max.cycles.only}{Logical. If TRUE, the fragments are only reported as parts of their longer cycle.}
}
\details{
A cycle can be part of larger cycle: if \code{max.cycles.only} all the cycles are reported but, if this parameter is True only the larger cycles are reported.
A warning recalls that for cycles k > 4 the fragments of a cycle are not necessarily all connected to each other (a fragment, due to its location in the original object, can only be connected to a limited number of adjacent fragments).
}
\value{
A data frame with the number of k-cycles for each k values in [3;k].
}
\author{Sebastien Plutniak <sebastien.plutniak at posteo.net>}

\seealso{
    \code{\link[igraph]{subgraph_isomorphisms}}
}

\examples{
g <- frag.simul.process(n.components=20, vertices=50, disturbance=.15)
frag.cycles(g, kmax=4, max.cycles.only=FALSE)
frag.cycles(g, kmax=4, max.cycles.only=TRUE)
}
\keyword{measurement}

\name{make_sr_graph}
\alias{make_sr_graph}
\alias{make_sr_graph-methods}
\title{
    Make a "similarity" relationships graph.
}
\description{
    Takes a \code{\link[=Frag.object-class]{frag.object}} and returns an undirected graph representing the "similarity" relationships between archaeological fragments. A "similarity" relationship  between fragments is defined if there is an acceptable likelihood that those fragments were part of the same object.
}
\usage{
make_sr_graph(object)
}
\arguments{
  \item{object}{
  A \code{\link[=Frag.object-class]{frag.object}} object.
}

}
\details{Returns an undirected graph of "\code{igraph}" class. The "fragments" data frame of the frag.object is used to set the vertices attributes.}
\value{
An undirected \code{igraph} class graph. The "frag_type" graph attribute is set with the "similarity" character value.
}
\author{Sebastien Plutniak <sebastien.plutniak at posteo.net>}

\seealso{\link{make_frag_object}}

\examples{
sr.df <- matrix( c(1,1, 9,1, 10,1, 11,2, 12,2, 13,2), ncol=2, byrow=TRUE)
fragments.df <- data.frame(1:13, letters[1:13]) 
crsr_g <- make_frag_object(sr=sr.df, fragments=fragments.df)
make_sr_graph(crsr_g)
}
\keyword{build data}

\name{FontJuvenal}
\docType{data}

\alias{FontJuvenal}
\alias{fontjuvenal.connection}
\alias{fontjuvenal.fragments}

\title{Dataset: Refitting relationships between pottery fragments from Font-Juvenal cave}
\description{
   The Font-Juvenal cave is a Neolithic archaeological site in France. This dataset describes the refitting 'connection' relationships between pottery fragments found during excavations. A connection relationship refers to a physical connection between two fragments that were part of the same object. 
    
    The dataset is composed of two tables:
    \itemize{
        \item\code{fontjuvenal.connection}: connection relationships between fragments.
        \item\code{fontjuvenal.fragments}: information about the fragments.
    }
}
\usage{data(FontJuvenal)}
\format{
    \itemize{
        \item\code{fontjuvenal.connection} is a 351x2 matrix. Each line describes a connection relationship between two fragments. Their  unique identifiers are given in column 'id1' and in column 'id2', respectively. 
        \item\code{fontjuvenal.fragments} is 354x4 data frame:
        \itemize{
            \item id:  fragment unique identifier 
            \item layer:  simplified stratigraphic layer
            \item layer.detailed:  stratigraphic layer, with detail about where the fragment was found (in the sedimentary layer, or in a hearth contained in the layer)
            \item square: square where the fragment was found
        }
    }
    
}
\references{
    \itemize{
		\item  Caro J. 2024. 'Font-Juvénal_Refiting', Zenodo, doi:  https://doi.org/10.5281/zenodo.14515444     
        }
    }
\keyword{datasets}

\examples{
data(FontJuvenal)
head(fontjuvenal.fragments)
}

\name{frag.get.parameters}
\alias{frag.get.parameters}

\title{Returns a series of descriptive statistics for a fragmentation graph}
\description{Returns a series of descriptive statistics for a fragmentation graph.}
\usage{frag.get.parameters(graph, layer.attr, verbose=TRUE)}

\arguments{
  \item{graph}{An \code{igraph} undirected graph.}
  \item{layer.attr}{Character. The name of the vertices attribute giving the spatial unit of each fragment (e.g. layer).}
  \item{verbose}{Logical. Whether to print or not warning messages.}
}
\details{
This function is a convenient function to get general information about a fragmentation graph. It is particularly useful for setting the parameters of the \code{frag.simul.process} function.
It returns:
\itemize{
  \item n.components: number of objects (i.e. connected components)
  \item vertices: number of fragments
  \item edges: number of connection relationships between fragments
  \item balance: estimated fragments balance, i.e. proportion of fragments in the first spatial unit (see details)
  \item components balance: estimated objects balance, i.e. proportion of objects in the first spatial unit (see details)
  \item disturbance: estimated disturbance, i.e. proportion of fragments which moved between spatial units (see details)
  \item aggreg.factor: aggregation factor, diversity of the components' edge count (see details)
  \item planar: whether the graph is planar or not
  \item edge.weights.sum: sum of the edge weights
  \item edge.weights.median: median of the edge weights
  \item edge.weights.median.abs.dev.: median absolute deviation of the edge weights
}

The (fragments) \bold{balance} and \bold{components balance} are calculated from the subgraph including only the fragments connected to fragments from the same spatial unit. This is to approximate these proportions of deposited materials regardless the effects of posterior disturbances. Results are given about the spatial unit whose label comes first alphanumerically.  

When studying a pair of spatial units, \bold{disturbance} is defined as the proportion of fragments which moved from their initial spatial unit to the other spatial unit. This definition is used in the \code{frag.process.simul} function to generate random fragmentation graphs.
However, \code{frag.get.parameters} uses a different definition since its aim is different: considering a fragmentation graph representing the result of unknown post-depositional processes, \code{frag.get.parameters} returns an estimation of the disturbance which might happened. This estimation is computed from a subgraph including only the components with fragments from the two spatial units: disturbance is get from the number of fragments belonging to the less represented spatial unit in this subgraph over the total number of fragments in the subgraph.

The \bold{aggregation factor} reflects the diversity of the components' edge count. The factor is calculated by: \deqn{1 - 1/(1 + sd(components\ edge\ count))}

The optional RBGL package is required to determine the planarity of the graph. If it is not installed, the \code{planar} argument is set to FALSE by default.
}
\value{
A list of parameters values (n.components, vertices, edges, balance, components.balance, disturbance, aggreg.factor, planar, edge.weights.sum, edge.weights.median, edge.weights.median.abs.dev.).
}
\author{Sebastien Plutniak <sebastien.plutniak at posteo.net>}

\seealso{
		 \link[archeofrag]{frag.get.layers.pair}, 
		 \link[archeofrag]{frag.simul.process}, 
		 \link[stats]{sd}, 
		 \link[stats]{median},
		 \link[stats]{mad}, 
		 \code{\link[RBGL]{boyerMyrvoldPlanarityTest}}
		 }

\examples{
g <- frag.simul.process(n.components=20, vertices=50, disturbance=0.1)
frag.get.parameters(g, "layer")
}
\keyword{summary}

\name{CuzoulSouth}
\docType{data}

\alias{CuzoulSouth}
\alias{cuzoul.south.connection}
\alias{cuzoul.south.fragments}

\title{Dataset: Refitting relationships between fauna fragments from the Cuzoul de Gramat site}
\description{
   The Cuzoul de Gramat site is a prehistoric site located in France. This dataset describes the refitting 'connection' relationships between bone fragments found during the excavation of Mesolithic layers from the 'South' sector of the site. Connection relationship refers to physical connection between two bone fragments that were part of the same object. 
    
    The dataset includes two tables:
    \itemize{
        \item\code{cuzoul.south.connection}: connection relationships between fragments.
        \item\code{cuzoul.south.fragments}: information about the fragments.
    }
}
\usage{data(CuzoulSouth)}
\format{
    \itemize{
        \item\code{cuzoul.south.connection} is a 135x2 matrix. Each line describes a connection relationship between two fragments. Their unique identifiers are given in column 'id1' and in column 'id2', respectively. 
        \item\code{cuzoul.south.fragments} is 233x9 data frame:
        \itemize{
            \item id: fragment unique identifier 
            \item layer: stratigraphic layer
            \item sub.layer: stratigraphic sub-layer
            \item thickness: thickness of the fragment (in centimetre)
            \item length: length of the fragment (in centimetre)
            \item surface: approximated surface of the fragment, product of the thickness by the length (in square centimeter)
            \item x: x coordinate of the grid square where the bone fragment was found (in centimetre)
            \item y: y coordinate of the grid square where the bone fragment was found (in centimetre)
            \item z: approximated z coordinate of the location where the bone fragment was found (in centimetre)
        }
    }
    
}
\references{
    \itemize{
		\item  Gardeur M. 2025. Bone refits from the Cuzoul de Gramat Mesolithic layers (archaeological site, France). Zenodo. https://doi.org/10.5281/zenodo.14975910
        }
    }
\keyword{datasets}

\examples{
data(CuzoulSouth)
head(cuzoul.south.fragments)
}

\name{archeofrag-package}
\alias{archeofrag-package}
\alias{archeofrag}
\docType{package}
\title{Archeofrag: Refitting and Spatial Analysis in Archaeology}
\description{Methods to analyse fragmented objects in archeology using refitting relationships between fragments scattered in archeological spatial units (e.g. stratigraphic layers). Graphs and graph theory are used to model archeological observations. The package is mainly based on the 'igraph' package for graph analysis. Functions can: 1) create, manipulate, and simulate fragmentation graphs, 2) measure the cohesion and admixture of archeological spatial units, and 3) characterise the topology of a specific set of refitting relationships. An empirical dataset is also provided as an example.
}
\author{Sebastien Plutniak
Maintainer: Sebastien Plutniak <sebastien.plutniak at posteo.net>
}
\keyword{package}
\seealso{
\code{\link[igraph:igraph-package]{igraph}},
\code{\link[RBGL:RBGL.overview]{RBGL}} 
}

\name{Chauzeys}
\docType{data}

\alias{Chauzeys}
\alias{chauzeys.connection}
\alias{chauzeys.fragments}

\title{Dataset: Refitting relationships between lithic fragments from the Chauzeys site}
\description{
   Chauzey is an open-air Palaeolithic site located in France. This dataset describes the refitting 'connection' relationships between lithic fragments found during excavations. Connection relationship refers to physical connection between two fragments that were part of the same object. 
    
    The dataset is composed of two tables:
    \itemize{
        \item\code{chauzeys.connection}: connection relationships between fragments.
        \item\code{chauzeys.fragments}: information about the fragments.
    }
}
\usage{data(Chauzeys)}
\format{
    \itemize{
        \item\code{chauzeys.connection} is a 1846x2 matrix. Each line describes a connection relationship between two fragments. Their unique identifiers are given in column 'id1' and in column 'id2', respectively. 
        \item\code{chauzeys.fragments} is a 2119x9 data frame:
        \itemize{
            \item id:  fragment unique identifier 
            \item layer:  chronocultural unit associated to the fragment was found. Values are numbered in expected chrono-stratigraphic order
            \item volume: estimated volume of the fragment computed as width by thickness by length
            \item width: width of the fragment (in millimeter)
            \item thickness: thickness  of the fragment (in millimeter)
            \item length: length of the fragment (in millimeter)
            \item x: x coordinate of the location where the lithic fragment was found (in metre)
            \item y: y coordinate of the location where the lithic fragment was found (in metre)
            \item z: z coordinate of the location where the lithic fragment was found (in metre)
        }
    }
    
}
\references{
    \itemize{
		\item Chadelle J.-P. (dir.), with M. Baillet, A. Barbeyron, M. Brenet, H. Camus, É. Claud, F. Jude, S. Kreutzer, A. Michel,  N. Mercier, M. Rabanit, S. Save, F. Sellami, A. Vaughan-Williams. 2021. Chauzeys, Saint-Médard-de-Mussidan (Dordogne, Nouvelle-Aquitaine), Rapport final d'opération archéologique, Périgueux, Conseil départemental de la Dordogne, Service départemental d'archéologie.
        }
    }
\keyword{datasets}

\examples{
data(Chauzeys)
head(chauzeys.fragments)
}

\name{frag.get.layers}
\alias{frag.get.layers}
\title{
Extracts the subgraph of each selected archaeological spatial unit.
}
\description{
Extracts the subgraph of each selected archaeological spatial unit (stratigraphic layer or any other type of spatial unit).
}
\usage{
frag.get.layers(graph, layer.attr, sel.layers)
}
\arguments{
  \item{graph}{An undirected \code{igraph} object.}
  \item{layer.attr}{Character. The name of the vertices attribute giving the spatial unit of each fragment.}
  \item{sel.layers}{Character. The identifier(s) of the stratigraphic spatial units to retrieve.}
}
\details{
This function is only a convenient function to extract the subgraphs of selected  archaeological spatial unit (stratigraphic layer or any other type of spatial unit).
A graph is created for each layer in the vertex attribute given by the \code{layer.attr} argument.
}
\value{
A list with a graph for each selected stratigraphic layer.
}
\author{Sebastien Plutniak <sebastien.plutniak at posteo.net>}

\examples{
g <- frag.simul.process(n.components=20, vertices=50, disturbance = .15)
igraph::V(g)$layers <- c(rep("layer1", 20), rep("layer2", 20), rep("layer3", 10))
frag.get.layers(g, layer.attr="layers", sel.layers=c("layer1", "layer2"))
}
\keyword{extraction}

\name{Grotte16}
\docType{data}

\alias{Grotte16}
\alias{grotte16.connection}
\alias{grotte16.fragments}

\title{Dataset: Refitting relationships between bone fragments from the Grotte 16 site, Palaeolithic levels}
\description{
Grotte 16 is a Palaeolithic site located in Cénac-et-Saint-Julien, France. This dataset describes the refitting 'connection' relationships between bone fragments found during the excavation of the Middle and Upper Palaeolithic levels led by Francois Leveque from 1976 to 1987. Connection relationship refers to physical connection between two fragments that were part of the same object. 
    
    The dataset includes two tables:
    \itemize{
        \item\code{grotte16.connection}: connection relationships between fragments.
        \item\code{grotte16.fragments}: information about the fragments.
    }
}
\usage{data(Grotte16)}
\format{
    \itemize{
        \item\code{grotte16.connection} is a 102x2 matrix. Each line describes a connection relationship between two fragments. Their unique identifiers are given in column 'id1' and in column 'id2', respectively. 
        \item\code{grotte16.fragments} is a 164x11 data frame:
        \itemize{
            \item id: fragment unique identifier 
            \item layers.at.excavation: archaeological layer 
            \item layer.uncorrected: layer where the fragment was found, as 
            \item layers.summarized: layer where the fragment was found, as attributed by F. Leveque
            \item layer.simplified: layer where the fragment was found, as attributed by F. Leveque
            \item square: square of the excavation grid where the bone fragment was found 
            \item square.x: X label of the excavation grid's square where the bone fragment was found 
            \item square.y: Y label of the excavation grid's square where the bone fragment was found 
            \item x: x coordinate of the location where the bone fragment was found (in centimetre)
            \item y: y coordinate of the location where the bone fragment was found (in centimetre)
            \item z: z coordinate of the location where the bone fragment was found (in centimetre)
            \item size: size class (in centimetre)
        }
    }
}


\details{
Refits were recorded by set of refitting fragments (and not by pair). Consequently, for the sets including more than two fragments, it is impossible to determine the exact distribution of the refitting relationships (if fragment A refits with fragments B and C, whether fragments B and C also refits together). So, in this dataset, the refits for those sets are parsimoniously modeled as chains (by consequence, the number of relationships might be underestimated).  
}

\references{
    \itemize{
        \item Dancette C., E. Discamps, S. Plutniak. 2025. 'Bone refits from the Grotte XVI Pleistocene Faunal Assemblage (Cénac-et-Saint-Julien, France)'. Zenodo. https://doi.org/10.5281/zenodo.15655628
        }
    }
\keyword{datasets}

\examples{
data(Grotte16)
head(grotte16.fragments)
}

\name{TaiSouth}
\docType{data}

\alias{TaiSouth}
\alias{tai.south.connection}
\alias{tai.south.fragments}

\title{Dataset: Refitting relationships between pottery fragments from the Tai site, South entrance sector}
\description{
    The Tai is a Neolithic archaeological site in France. Three sectors were distinguished: the North entrance, the South entrance and the Cave. This dataset regards the refitting 'connection' relationships between pottery fragments found during excavations led by Claire Manen in the South entrance sector. A connection relationship refers to a physical connection between two fragments that were part of the same object. 
    
    The dataset includes two tables:
    \itemize{
        \item\code{tai.south.connection}: connection relationships between fragments.
        \item\code{tai.south.fragments}: information about the fragments.
    }
}
\usage{data(TaiSouth)}
\format{
    \itemize{
        \item\code{tai.south.connection} is a 79x2 matrix. Each line describes a connection relationship between two fragments. Their unique identifiers are given in column 'id1' and in column 'id2', respectively. 
        \item\code{tai.south.fragments} is 82x7 data frame:
        \itemize{
            \item id:  fragment unique identifier 
            \item layer:   stratigraphic layer
            \item sherd_size: size class of the pottery sherd (nb: the numeric value is not metrical)
            \item sherd_thickness: thickness of the pottery sherd in millimeter
            \item x: x coordinates of the location where the pottery sherd was found (in centimetres)
            \item y: y coordinates of the location where the pottery sherd was found (in centimetres)
            \item z: z coordinates of the location where the pottery sherd was found (in centimetres)
        }
    }
    
}
\references{
    \itemize{
		\item  Caro J., Plutniak S. 2022. 'Refitting and Matching Neolithic Pottery Fragments from the Taï site, France'. Zenodo, doi: https://doi.org/10.5281/zenodo.7408706
        }
        
    }
\keyword{datasets}

\examples{
data(TaiSouth)
head(tai.south.fragments)
}

\name{frag.layers.cohesion}
\alias{frag.layers.cohesion}

\title{Cohesion measure of layers}
\description{Returns the cohesion value of two archaeological spatial units (layers or any other type of spatial units).}
\usage{frag.layers.cohesion(graph, layer.attr, morphometry, x, y, z, verbose=TRUE)}

\arguments{
  \item{graph}{An undirected \code{igraph} object.} 
  \item{layer.attr}{Character. The name of the vertices attribute giving the spatial unit of the fragments.}
  \item{morphometry}{Character. Optional, to pass to the `frag.edges.weighting` function: name of the vertex attribute with the morphometric value of the fragments.}
  \item{x}{Character. Optional, to pass to the `frag.edges.weighting` function: name of the vertex attribute with the "x" coordinate of the fragments.}
  \item{y}{Character. Optional, to pass to the `frag.edges.weighting` function: name of the vertex attribute with the "y" coordinate of the fragments.}
  \item{z}{Character. Optional, to pass to the `frag.edges.weighting` function:  name of the vertex attribute with the "z" coordinate of the fragments.}
  \item{verbose}{Logical. Whether to print or not warning messages.}
}
\details{
Note that this function  must be used after weighting the edges with \code{frag.edges.weighting}.

The cohesion value of a spatial unit  is computed as: 

	\deqn{\frac{V_{unit_i} + W_{unit_i}}{ \sum_{j=1}^{2} V_{unit_j} + W_{unit_j}} }
	
with V the number of vertices in the unit and W the sum of the edge weights within the unit.

The measure takes into account the balance between the information about each spatial unit.
Results range in ]0;1[, with 0 for two spatial units with only inter-spatial units connection relationships, and 1 if there are not inter-spatial units relationships and a  significant imbalance of information on the two spatial units.

The basic use of this function is intended for a graph with two spatial units, whose edges have been previously weighted using the \code{frag.edges.weighting} function (an error message is displayed if the vertice attribute "layer" contains less than two spatial units, and if the graph does not have an edge attribute "weight").

However, this function can also be used for a graph with more than two spatial units. In this case, a subgraph is generated for each pair of spatial units (using the \code{frag.get.layers.pair} function), the \code{frag.edges.weighting} function is applied to weight their edges (a warning message is displayed), and the cohesion is computed.
}
\value{
If the graph has only two spatial units, the function returns a numeric vector with a cohesion value (]0;1[) for each spatial units (sorted in alphanumerical order).
If the graph has more than two spatial units, the function returns a matrix with a row corresponding to the comparison of two spatial units, two columns including the cohesion values, and row names giving the labels of the spatial units (separated by the sign '/').
}
\author{Sebastien Plutniak <sebastien.plutniak at posteo.net>}

\seealso{
\code{\link[archeofrag]{frag.edges.weighting}},
\code{\link[archeofrag]{frag.get.layers.pair}}
}
\examples{
g <- frag.simul.process(n.components=20, vertices=50, disturbance=.1)
frag.layers.cohesion(g, layer.attr="layer")
}
 
\keyword{measurement}

\name{frag.graph.reduce}
\alias{frag.graph.reduce}

\title{Reduce the size of a fragmentation graph}
\description{Remove fragments from a fragmentation graph}

\usage{frag.graph.reduce(graph=NULL, n.frag.to.remove=NULL, conserve.objects.nr=FALSE,
       conserve.fragments.balance=FALSE, conserve.inter.units.connection=FALSE,
       verbose=FALSE)}

\arguments{
  \item{graph}{An \code{igraph} object.} 
  \item{n.frag.to.remove}{Integer. Number of fragments (i.e. vertices) to remove.}
  \item{conserve.objects.nr}{Logical. If TRUE, preserve the number of objects (i.e. connected components) in the graph.}
  \item{conserve.fragments.balance}{Logical. If TRUE, try to preserve the proportion of fragments in the first and second spatial units ('balance').}
  \item{conserve.inter.units.connection}{Logical. If TRUE, preserve the proportion of connection relationships between fragments located in different spatial units (see details).}
  \item{verbose}{Logical. Whether to print or not warning messages.}
  
}
\details{
This function reduces the number of fragments in a fragmentation graph, while controlling that no singleton are created, and (optionally) that the output graph and the input graph have the same number of connected components. Note that if 'conserve.objects.nr' is TRUE then the reduction process might stop before reaching the number of fragments to remove (when the graph only contains pairs of fragments). 
Similarly, if 'conserve.fragments.balance' is TRUE, then the result is as close as possible from the initial proportion of fragments (i.e. 'frag.get.parameters()' 'balance' output value). 
Finally, if 'conserve.inter.units.connection' is TRUE, the proportion of connection relationships (i.e. graph edges) between fragments from different spatial units to conserve is equal to the proportion of fragments (i.e. vertices) to preserve.
}
\value{
A fragmentation graph (igraph object) with less fragments (vertices).
}
\author{Sebastien Plutniak <sebastien.plutniak at posteo.net>}
\seealso{
    \code{\link[archeofrag]{frag.get.parameters}}
    \code{\link[archeofrag]{frag.get.layers.pair}}
}

\examples{
g <- frag.simul.process(n.components=15, vertices=50, disturbance=.15) 
igraph::gorder(g)
igraph::components(g)$no
# reduce the number of fragments and conserve the number of connected components:
g1 <- frag.graph.reduce(g, n.frag.to.remove = 40, conserve.objects.nr = TRUE)
igraph::gorder(g1)
igraph::components(g1)$no
# reduce the number of fragments and do not conserve the number of connected components:
g2 <- frag.graph.reduce(g, n.frag.to.remove = 40, conserve.objects.nr = FALSE)
igraph::gorder(g2)
igraph::components(g2)$no
}
 
\keyword{simulation}

\name{make_crsr_graph}
\alias{make_crsr_graph}
\title{
    Makes a "connection" relationships graph including the "similarity" relationships.
}
\description{
    Takes a \code{\link[=Frag.object-class]{frag.object}} in argument and returns an undirected graph representing the relationships between archaeological fragments. "Connection" and "similiarity" relationships are combined. A "connection" relationship refers to a physical connection between two fragments that were part of the same object. A "similarity" relationship between fragments is defined if there is an acceptable likelihood that those fragments were part of the same object.
}
\usage{
    make_crsr_graph(object)
}
\arguments{
  \item{object}{A \code{\link[=Frag.object-class]{frag.object}}.}

}
\details{A complementary function to the \code{\link{make_cr_graph}} function. This function handles both the "connection" and "similarity" relationships.  This can be useful, given that "similarity" relations are more frequently documented in archaeological datasets than the "connection" relationships. 

The function returns an undirected graph of "\code{igraph}" class, using the "fragments" data frame of the frag.object to set the vertices attributes. 

Both "connection" and "similarity" relationships are included in the resulting graph. The edge attribute "type_relation" is set with a character "cr" value for "connection" relationships and with "sr" for "similarity" relationships. 
Edge weights are not set by this function, and it is recommended to use the \code{frag.edges.weighting} function. A "frag_type" graph attribute is set with a "connection and similarity" value.
}

\value{An undirected "igraph" class graph.}
\author{Sebastien Plutniak <sebastien.plutniak at posteo.net>}

\seealso{\link{make_frag_object}, \link{make_cr_graph}, \link{make_sr_graph}}

\examples{
cr.df <- matrix(c(1,2, 1,3, 2,3, 4,5, 4,6, 7,8), ncol=2, byrow=TRUE)
sr.df <- matrix( c(1,1, 9,1, 10,1, 11,2, 12,2, 13,2), ncol=2, byrow=TRUE)
fragments.df <- data.frame(1:13, letters[1:13]) 
crsr_g <- make_frag_object(cr=cr.df, sr=sr.df, fragments=fragments.df)
make_crsr_graph(crsr_g)
}
\keyword{build data}

\name{frag.diameters}
\alias{frag.diameters}
\title{Diameter distribution for unconnected graphs}
\description{
Returns the distribution of the diameter values of a fragmentation graph.
}
\usage{
frag.diameters(graph, cumulative = FALSE)
}
\arguments{
  \item{graph}{An \code{igraph} object.}
  \item{cumulative}{Logical. If TRUE the cumulative relative frequency of the diameters is reported.}
}
\details{
\code{frag.diameters} wraps the \code{igraph} \code{diameter} function.
For graphs representing the fragmentation of archeological objects, the diameter of each component of the graph (i.e. archeological objects) can be interpreted:
as a measure of the intensity of fragmentation (when all the fragments of the initial object are known);
as a measure of the scattering of the fragments (when not all the fragments are known);
}
\value{
A numeric vector of the length equal to the maximum diameter value found. The first element is the frequency of the diameter values = 1, the second element is the frequency of diameter values = 2, etc. If \code{cumulative} is True, the cumulative density is returned.
}
\author{Sebastien Plutniak <sebastien.plutniak at posteo.net>}

\seealso{
    \code{\link[igraph]{diameter}}
}
\examples{
g <- frag.simul.process(n.components=20, vertices=50, disturbance=.15)
frag.diameters(g)
frag.diameters(g, cumulative=TRUE)

}

\keyword{measurement}

\name{Frag.object-class}
\Rdversion{1.1}
\docType{class}
\alias{Frag.object-class}
\alias{make_cr_graph,Frag.object-method}
\alias{make_crsr_graph,Frag.object-method}
\alias{make_sr_graph,Frag.object-method}
\alias{show,Frag.object-method}

\title{Class \code{"Frag.object"}}
\description{
    A class for archaeological "fragmentation" datasets. This class construction aims to ensure that the data have been properly built before performing the next steps of the analysis.
    A convenient constructor function, \link{make_frag_object}, is provided.
    
}
\section{Objects from the Class}{
    Objects can be created by calls of the form:
    
    \code{make_frag_object(cr, sr, fragments)}.
}
\section{Slots}{
  \describe{
    \item{\code{df.cr}:}{Object of class \code{"matrix"} (\code{"data.frame"} are allowed and automatically converted)}
    \item{\code{df.sr}:}{Object of class \code{"matrix"}  (\code{"data.frame"} are allowed and automatically converted)}
    \item{\code{fragments.df}:}{Object of class \code{"data.frame"}}
    \item{\code{frag_type}:}{Object of class \code{"character"} }
  }
}
\section{Methods}{
  \describe{
    \item{make_cr_graph}{\code{signature(object = "Frag.object")}:  Makes an undirected graph representing the "connection" relationships between archaeological fragments.}
    \item{make_sr_graph}{\code{signature(object = "Frag.object")}:  Makes an undirected graph representing the "similarity" relationships between archaeological fragments.}
    \item{make_crsr_graph}{\code{signature(object = "Frag.object")}: Makes an undirected graph combining the "connection" and "similarity" relations between archaeological fragments.}
    \item{show}{\code{signature(object = "Frag.object")}: \code{show} method for Frag.object }
  }
}

\author{Sebastien Plutniak <sebastien.plutniak at posteo.net>}

\seealso{
    \code{\link[archeofrag]{make_frag_object}},
    \code{\link[archeofrag]{make_cr_graph}},
    \code{\link[archeofrag]{make_sr_graph}},
    \code{\link[archeofrag]{make_crsr_graph}}
}

\examples{
showClass("Frag.object")
}
\keyword{build data}

\name{StCesaire1987}
\docType{data}

\alias{StCesaire1987}
\alias{st.cesaire.1987.connection}
\alias{st.cesaire.1987.fragments}

\title{Dataset: Refitting relationships between bone fragments from the Saint Césaire site, Palaeolithic levels}
\description{
La Roche a Pierrot is a Palaeolithic rock shelter located in Saint Césaire, France. This dataset describes the refitting 'connection' relationships between bone fragments found during the excavation of the Middle and Upper Palaeolithic levels led by Francois Leveque from 1976 to 1987. Connection relationship refers to physical connection between two fragments that were part of the same object. 
    
    The dataset includes two tables:
    \itemize{
        \item\code{st.cesaire.1987.connection}: connection relationships between fragments.
        \item\code{st.cesaire.1987.fragments}: information about the fragments.
    }
}
\usage{data(StCesaire1987)}
\format{
    \itemize{
        \item\code{st.cesaire.1987.connection} is a 1188x2 matrix. Each line describes a connection relationship between two fragments. Their unique identifiers are given in column 'id1' and in column 'id2', respectively. 
        \item\code{st.cesaire.1987.fragments} is a 2006x9 data frame:
        \itemize{
            \item id: fragment unique identifier 
            \item layer: archaeological layer where the fragment was found, after revision
            \item layer.uncorrected: layer  where the fragment was found, as attributed by F. Leveque
            \item spit: minimal and maximal depth of the excavation spit in which the bone fragment was found 
            \item x: x coordinate of the location where the bone fragment was found (in centimetre)
            \item y: y coordinate of the location where the bone fragment was found (in centimetre)
            \item z: z coordinate of the location where the bone fragment was found (in centimetre)
            \item square: square where the fragment was found
            \item length: length of the fragment (in millimetre)
        }
    }
}
\details{
Refits were recorded by set of refitting fragments (and not by pair). Consequently, for the sets including more than two fragments, it is impossible to determine the exact distribution of the refitting relationships (if fragment A refits with fragments B and C, whether fragments B and C also refits together). So, in this dataset, the refits for those sets are parsimoniously modeled as chains (by consequence, the number of relationships might be underestimated).  
}

\references{
    \itemize{
		\item Morin E., S. Plutniak. 2025. 'Middle and Upper Palaeolithic Bone Refitting data from La Roche a Pierrot site (Saint-Cesaire, France), Excavations 1976-1987'. Zenodo. https://doi.org/10.5281/zenodo.15638561
        }
    }
\keyword{datasets}

\examples{
data(StCesaire1987)
head(st.cesaire.1987.fragments)
}

\name{frag.path.lengths}
\alias{frag.path.lengths}

\title{Path length distribution for unconnected graphs}
\description{Path length distribution for unconnected graphs}

\usage{frag.path.lengths(graph, cumulative=FALSE)}

\arguments{
  \item{graph}{An \code{igraph} object.} 
  \item{cumulative}{Logical. If TRUE, the cumulative relative frequency of the path lengths is returned.}
}
\details{
This function is a wrapper of \code{igraph} \code{distance_table} returning the frequency of path lengths in undirected and unconnected graphs. In the context of archaeological fragmentation analysis, path lengths are interpreted to characterise the properties of fragmentation within a layer.
}
\value{
A numeric vector having the same length as the maximum path length. The first element of the vector is the frequency of the paths of length 1, the second element is the frequency of the paths of length 2, etc. 
}
\author{Sebastien Plutniak <sebastien.plutniak at posteo.net>}

\seealso{
    \code{\link[igraph]{distance_table}}
}
\examples{
g <- frag.simul.process(n.components=20, vertices=50, disturbance = .15)
frag.path.lengths(g)
frag.path.lengths(g, cumulative=TRUE)
}
 
\keyword{measurement}

\name{frag.graph.plot}
\alias{frag.graph.plot}

\title{Plot a fragmentation graph}
\description{A function to plot the graph made by the \code{archeofrag} package. }
\usage{frag.graph.plot(graph, layer.attr, ...)}

\arguments{
  \item{graph}{An \code{igraph} undirected object with a "frag_type" attribute.} 
  \item{layer.attr}{Character. The name of the vertices attribute giving the spatial unit of each fragment.}
  \item{...}{Optional arguments sent to \code{plot.igraph}.}
}
\details{
This function is a wrapper for the \code{plot.igraph} method for igraph objects. The layout is computed using the fruchterman-reingold algorithm, with some changes as a function of the value of the "frag_type" graph attribute.
For graphs including similarity relations, \code{igraph}' \code{component_wise} layout modifier is applied.
For graphs with connection and similarity relationships, the edges for connection relations are coloured in green.
For graphs with connection relationships only and two spatial units, the vertices are placed on the top (spatial unit 1) and bottom (spatial unit 2) part of the plot.

}
\value{
    Returns \code{NULL} and plot the graph.
}
\author{Sebastien Plutniak <sebastien.plutniak at posteo.net>}

\seealso{
  \link[igraph]{plot.igraph}, \link[igraph]{component_wise}, \link[igraph]{layout_with_fr}
}
\examples{
g <- frag.simul.process(n.components=20, vertices=50, disturbance=.15)
frag.graph.plot(g, "layer")
}
 
\keyword{summary}

\name{make_cr_graph}
\alias{make_cr_graph}
\alias{make_cr_graph-methods}
\title{
    Make a "connection" relationships graph.
}
\description{
    Takes a \code{\link[=Frag.object-class]{frag.object}} and returns an undirected graph representing the "connection" relationships between archaeological fragments. A "connection" relationship refers to a physical connection between two fragments that were part of the same object.
}
\usage{
make_cr_graph(object)
}
\arguments{
  \item{object}{
  A \code{\link[=Frag.object-class]{frag.object}} object.
}

}
\details{Returns an undirected graph of "\code{igraph}" class. The "fragments" data frame of the frag.object is used to set the vertices attributes.}
\value{
An undirected \code{igraph} class graph. The "frag_type" graph attribute is set with the "connection" character value.
}
\author{Sebastien Plutniak <sebastien.plutniak at posteo.net>}

\seealso{\link{make_frag_object}}
\examples{
cr.df <- matrix(c(1,2, 1,3, 2,3, 4,5, 4,6, 7,8), ncol=2, byrow=TRUE)
sr.df <- matrix( c(1,1, 9,1, 10,1, 11,2, 12,2, 13,2), ncol=2, byrow=TRUE)
fragments.df <- data.frame(1:13, letters[1:13]) 

cr_g <- make_frag_object(cr=cr.df, fragments=fragments.df)
crsr_g <- make_frag_object(cr=cr.df, sr=sr.df, fragments=fragments.df)

make_cr_graph(cr_g)
make_cr_graph(crsr_g)
}
\keyword{build data}

\name{frag.simul.compare}
\alias{frag.simul.compare}

\title{From an observed fragmentation graph, simulates two series of graphs corresponding to two deposition hypotheses.}
\description{Given an observed fragmentation graph, simulates two series of graphs corresponding to two deposition hypotheses, compares their properties and returns a summary table.}
\usage{frag.simul.compare(graph, layer.attr, iter, summarise=TRUE, verbose=TRUE, ...)}

\arguments{
  \item{graph}{An undirected \code{igraph} object. The 'observed' graph to compare to simulated graphs.} 
  \item{layer.attr}{Character. The name of the vertices attribute giving the layer of the fragments.}
  \item{iter}{Numerical. The number of simulated graphs to generate for each hypothesis (minimal value: 30).}
  \item{summarise}{Logical. Whether to report a comparative summary of the results.}
  \item{verbose}{Logical. Whether to print or not warning messages.}
  \item{...}{Further arguments passed to the `frag.simul.process` function.}
}
\details{
This function is a convenient wrapper integrating several functions of the \code{archeofrag} package to compare an observed fragmentation graph to similar simulated graphs. The \code{frag.simul.process} is used to generate two series of graphs from the properties of the observed graph: the first series is generated under the formation hypothesis H1 (one initial spatial unit) and the second series is generated under the hypothesis H2 (two initial spatial units). 
The edge count, edge weights sum, balance, components balance, disturbance, admixture, and cohesion values of the generated graphs are measured.

By default, the results are post-processed with the \code{frag.simul.summarise} function and a summary data frame is printed and included in the list of results which is silently returned. If the \code{summarise} parameter is set to FALSE, then the function returns a list of two data frames containing the numeric values measured for H1 and H2.


}
\value{
A named list with three items: 
"h1.data", a data frame with the numerical values measured on the graphs generated for H1;
"h2.data", a data frame with the numerical values measured on the graphs generated for H2;
"summary", a data frame summarising the comparison between the results for the two hypotheses and the values measured on the empirical graph.

}
\author{Sebastien Plutniak <sebastien.plutniak at posteo.net>}

\seealso{
\code{\link[archeofrag]{frag.simul.process}},
\code{\link[archeofrag]{frag.simul.summarise}}
}
\examples{
g <- frag.simul.process(n.components=20, vertices=50, disturbance=.15)
\dontrun{frag.simul.compare(g, layer.attr="layer", iter=30)}
}
 
\keyword{simulation}

\name{TaiCave}
\docType{data}

\alias{TaiCave}
\alias{tai.cave.connection}
\alias{tai.cave.fragments}

\title{Dataset: Refitting relationships between pottery fragments from the Tai site, Cave sector}
\description{
    The Tai is a Neolithic archaeological site in France. Three sectors were distinguished: the North entrance, the South entrance and the Cave. This dataset regards the refitting 'connection' relationships between pottery fragments found during excavations led by Claire Manen in the Cave sector. A connection relationship refers to a physical connection between two fragments that were part of the same object. 
    
    The dataset includes two tables:
    \itemize{
        \item\code{tai.cave.connection}: connection relationships between fragments.
        \item\code{tai.cave.fragments}: information about the fragments.
    }
}
\usage{data(TaiCave)}
\format{
    \itemize{
        \item\code{tai.cave.connection} is a 91x2 matrix. Each line describes a connection relationship between two fragments. Their unique identifiers are given in column 'id1' and in column 'id2', respectively. 
        \item\code{tai.cave.fragments} is 112x7 data frame:
        \itemize{
            \item id:  fragment unique identifier 
            \item layer:   stratigraphic layer
            \item sherd_size: size class of the pottery sherd (nb: the numeric value is not metrical)
            \item sherd_thickness: thickness of the pottery sherd in millimeter
            \item x: x coordinates of the location where the pottery sherd was found (in centimetres)
            \item y: y coordinates of the location where the pottery sherd was found (in centimetres)
            \item z: z coordinates of the location where the pottery sherd was found (in centimetres)
        }
    }
    
}
\references{
    \itemize{
		\item  Caro J., Plutniak S. 2022. 'Refitting and Matching Neolithic Pottery Fragments from the Taï site, France'. Zenodo, doi: https://doi.org/10.5281/zenodo.7408706
        }
        
    }
\keyword{datasets}

\examples{
data(TaiCave)
head(tai.cave.fragments)
}

\name{Eaton}
\docType{data}

\alias{Eaton}
\alias{eaton.connection}
\alias{eaton.fragments}

\title{Dataset: Refitting relationships between lithic fragments from the Eaton site}
\description{
   Eaton site is a 16th century Iroquoian village in western New York State, USA. This dataset describes the refitting 'connection' relationships between fragments of lithic points found during excavations led by William Engelbrecht. Connection relationship refers to physical connection between two fragments that were part of the same object. 
    
    The dataset is composed of two tables:
    \itemize{
        \item\code{eaton.connection}: connection relationships between fragments.
        \item\code{eaton.fragments}: information about the fragments.
    }
}
\usage{data(Eaton)}
\format{
    \itemize{
        \item\code{eaton.connection} is a 115x2 matrix. Each line describes a connection relationship between two fragments. Their unique identifiers are given in column 'id1' and in column 'id2', respectively. 
        \item\code{eaton.fragments} is 229x5 data frame:
        \itemize{
            \item id:  fragment unique identifier 
            \item x: x label of the grid square where the lithic fragment was found (square size is 2m x 2m)
            \item y: y label of the grid square where the lithic fragment was found (square size is 2m x 2m)
            \item feature: archaeological feature associated with the fragment is associated ('long house', 'midden', or 'outside').
            \item level: archaeological level where the fragment was found (0 to 5, from top soil to bottom).
        }
    }
    
}
\references{
    \itemize{
		\item Engelbrecht W. 2014. "Madison Point Refits", tDAR, doi: 10.6067/xcv8t43v1j 
        \item Plutniak S. 2025. "Reprocessing script for William Engelbrecht's 'Madison Point Refits' dataset (including generated tables and figures)". Zenodo. doi: 10.5281/zenodo.15091301
        }
    }
\keyword{datasets}

\examples{
data(Eaton)
head(eaton.fragments)
}
